#######################################################################
# Copyright: DAASI International GmbH 2018. All rights reserved.
#
# This is Open Source Software
# License: Artistic License 2.0
#
# Author: Peter Gietz, DAASI International GmbH, www.daasi.de
# For questions please mail to peter.gietz@daasi.de
#######################################################################
# 15.11.2005 KPG

package DAASIlib::CONF;

use strict;
use warnings;

use IO::Prompt;

use vars qw($VERSION);
use Log::Log4perl qw(:levels);

our $VERSION = "0.7";
our $ISDEBUG=0;
our $ISWEB = 0;

#use DAASIlib::LOG;
### test lin
use DAASIlib::Data;
my $data = new DAASIlib::Data;
use DAASIlib::Gettext;
my $gt=new DAASIlib::Gettext;

use Config::General;
use Getopt::Std;
use Text::Wrap;
#use Text::Autoformat;
use File::Basename;
use File::Copy;

use constant LINELEN=> 80;

sub new
{
    my $class;
    my $self;

    my ( $this , $param ) = @_;
    $class = ref($this) || $this;

    $self={
	data => {},
	logs =>[],
    };

    bless($self , $class );

    return $self;
}

sub whichConfigFile {
   my $self = shift;

   $self->{data}{configfile};

}


sub addConfig {
    my $self = shift;
    my ( $filename, $progpath, $etcdir, $addtoetc, $store,
	 $preferredlanguage, $is_taint, $default_lang ) = @_;

#    $self->log('debug', "addConfig(): called with filename $filename, progpath $progpath, etcdir $etcdir, addtoetc $addtoetc, store $store, preferredlanguage $preferredlanguage, is_taint $is_taint, default_lang $default_lang");

    my $fullconfigfilename = $self->getPathRight($filename,
						 $progpath, $etcdir,
						 $addtoetc);
#    $self->log('debug', "addConfig(): loading config $fullconfigfilename to $store");
    $self->newConf( $fullconfigfilename, $self->{data}, "yes", undef, $etcdir,
		    $preferredlanguage, $is_taint, $default_lang );
}

sub reloadConfig {
    my $self = shift;
    my ( $filename, $progpath, $etcdir, $addtoetc, $store,
	 $preferredlanguage, $is_taint, $oldconf, $default_lang  ) = @_;

    my $fullconfigfilename = $self->getPathRight($filename,
						 $progpath, $etcdir,
						 $addtoetc);
#    print "loading config   $fullconfigfilename to $store\n";

##    $self->newConf( $fullconfigfilename, $oldconf, "yes", undef, $etcdir,
    $self->newConf( $fullconfigfilename, $self->{data}, "yes", undef, $etcdir,
		    $preferredlanguage, $is_taint, $default_lang, 1 );
}

sub loadConfig {
    my $self = shift;
    my ( $filename, $progpath, $etcdir, $addtoetc, $progname, $isweb,
	 $preferredlanguage, $is_taint, $default_lang ) = @_;

    $ISWEB = 1 if $isweb;

    my %confall=();

    # for any case the separator is needed before defaults are loaded:
    $self->{data}{separator} = ',';

#    print "Loading configfile $filename\n";
    $self->storeLogMessage('debug', "Loading configfile $filename");
    $self->newConf( $filename, \%confall, "no", $progname, $etcdir,
		    $preferredlanguage, $is_taint, $default_lang  );

    $self->checkOpts();
    $self->setConfDefaults($progname );

    $self->getOptions();


    my $lang =  $self->getOptValue('language');
#    print "lang: $lang\n";
    if ( $lang ) {
	$gt->setlanguage($lang);
	my $pofile = './etc/locale/de_DE/LC_MESSAGES/'.$progname.'.po';
	$gt->init($pofile,$lang);

	$self->{gt} = $gt;
    }

    if ($self->getOptValue('debugmode') ) {
	$self->storeLogMessage('debug', "turning debugmode on");
	$ISDEBUG = 1;
    }

    $self->formatManPage($lang);


    if ( $self->getOptValue('printhelp') ) {
	print @{$self->{data}{manpage}};
	exit;
    }

#    print "Helpfeature: ",$self->getOptValue('helpfeature'),"\n";

    my $feature = $self->getOptValue('helpfeature');
    if ( $feature ) {
	if (length ($feature) == 1 ) {
	    if ( $self->{data}{usage}{$feature} ) {
		print "Feature:\n".
		    "$self->{data}{usage}{$feature}\n";
	    }
	    else {
		print "   Unknown feature $feature\n";
	    }
#	} elsif ($feature eq "functions") {
#	    $self->printFunctions;
	} elsif ( $feature =~ /^SUM/ ){
#	    print grep {/^\s\-\w /} @{$self->{data}{manpage} }[0];
	    print join "\n", sort { lc($a) cmp lc($b) } @{$self->{data}{summary} };
	} else {
	    if ( $self->{data}{options}{$feature}{key} ) {
		print "Feature:\n".
		    "$self->{data}{usage}{$self->{data}{options}{$feature}{key}}\n";
	    }
	    else {
		print "   Unknown feature $feature\n";
	    }
	}
	print "\n";
	exit;
    }

    if ( my $value = $self->getOptValue('configfile') ) {
	$self->{data}{configfile} = $value;
    }

    $self->{data}{progpath} = $progpath;
    $self->{data}{etcdir} = $etcdir;

    my $fullconfigfilename = $self->getPathRight($self->{data}{configfile},
						 $progpath, $etcdir,
						 $addtoetc);

#    print "fullconfigname: $fullconfigfilename\n";
#    exit();
    $self->storeLogMessage('debug', "Loading configfile $fullconfigfilename");
    $self->newConf( $fullconfigfilename, $self->{data}, 'yes', $progname,
		    $etcdir, $preferredlanguage, $is_taint, $default_lang );

    $self->loadParams();

    if ($self->is_debug) {
	$self->printConf("config after processing params:");
    }

    my $errors =0;

    $errors = $self->checkNotyet();

    if ( ! $errors ) {
	if ( ! $self->{data}{executetool} ) {
	    $errors = $self->checkDependencies ();
	    $errors += $self->checkAllValues ( );
	    $errors += $self->checkArgType ($errors);
	}
	else {
	    $errors = $self->checkValues('executetool');
	    $errors += $self->checkArgType ( $errors, 'mappingfile', 'inputfile', 'SQLfilename', 'SQLstring', 'language' );
	}
    }

    if ($self->is_debug) {
	$self->printConf("config after loading all:");
    }


     if ($errors) {
	print "Found $errors errors\n";
	print "\tbeware: wrong values come either from command line flags or",
	"\n\t        from the user config file $self->{data}{configfile}\n";
	print "\tFor more information type $self->{data}{progname}.pl".
	    " -$self->{data}{options}{'printhelp'}{key}\n\n";
	die "Aborting program\n\n";
    }


    my $binerrors += $self->checkPrograms();

    if ($binerrors) {
	print "Found $binerrors missing binaries\n";
	die "Aborting program\n\n";
    }

    my $processerrors += $self->checkProcesses();

    if ($processerrors) {
	print "Found $processerrors missing processes\n";
	die "Aborting program\n\n";
    }

#    if ($self->debug()) {
#    }

    $self->addFunctionParameters;


    ## now let us modify the loglevel for Log4Perl
    ##   print "setting loglevel: $self->{data}{loglevel}\n";

    if ( $self->{data}{logfile} ) {
	$self->{data}{loglevel} = $self->setLogLevel($self->{data}{loglevel});
    }

    return $gt;

}


sub checkPrograms {
    my $self = shift;
    my $errors = 0;

    foreach my $progName ( sort keys %{$self->{data}{checkprograms}} ) {
	my $bin = $self->{data}{checkprograms}{$progName}{name};
	my $must = $self->{data}{checkprograms}{$progName}{must};
 #       print "testing |$bin|\n";
	my $sysresult = `which $bin 2>/dev/null`;

	if ( $sysresult ) {
	    chomp($sysresult);
	    $self->{data}{checkprograms}{$progName}{bin} = $sysresult;
	}
	elsif ($must) {
	    print "ERROR: Couldn't find needed binary $bin\n";
	    $errors ++;
	}
	else  {
	    $self->storeLogMessage('warn',
				   "Couldn't find wanted binary $bin\n");
	}
    }
    return ( $errors);
}

sub checkProcesses {
    my $self = shift;
    my $errors = 0;

    foreach my $procName ( sort keys %{$self->{data}{checkprocesses}} ) {
	my $proc = $self->{data}{checkprocesses}{$procName}{name};
	my $must = $self->{data}{checkprocesses}{$procName}{must};
#        print "testing $procName |$proc|\n";
	my $sysresult = `ps aux | grep $proc 2>/dev/null`;

	my @proctab = split /\n/, $sysresult;

	my $is_process = 0;

	foreach my $procline ( @proctab ) {
	    if ( $procline !~ /grep $proc/ ) {
		$is_process ++;
	    }
	}

	if ( $is_process ) {
	    $self->storeLogMessage('debug',
		"Found $is_process processes of type $procName ($proc)\n")
		if $self->is_verbose();
	}
	elsif ($must) {
	    print "ERROR: Couldn't find needed process $procName ($proc)\n";
	    $errors ++;
	}
	else  {
	    $self->storeLogMessage('warn',
			"Couldn't find needed process $procName ($proc)\n");
	}
    }
    return ( $errors);
}


sub is_debug {
    my $self = shift;
    $ISDEBUG;
}

sub is_verbose {
    my $self = shift;

    $self->{data}{verbose};
}


sub setLogLevel {
    my $self = shift;
    my ( $level ) = @_;

    my $loglevel;

    $level = lc($level);

    if ( $level eq "no" ) { $loglevel = $OFF }
    elsif ( $level eq "all" ) { $loglevel = $ALL }
    elsif ( $level eq "debug" ) { $loglevel = $DEBUG }
    elsif ( $level eq "private" ) { $loglevel = $Log::Log4perl::PRIVATE_INT }
    elsif ( $level eq "info" ) { $loglevel = $INFO }
    elsif ( $level eq "warn" ) { $loglevel = $WARN }
    elsif ( $level eq "error" ) { $loglevel = $ERROR }
    elsif ( $level eq "fatal" ) { $loglevel = $FATAL }
    else { $loglevel = $OFF }

    return ($loglevel);
}

sub storeLogMessage {
    my $self = shift;
    my ($level, $message) = @_;

    # 2008/12/22 22:04:43
    my ( $sek, $min, $hour, $day, $mon, $year ) = localtime();
    my $timestamp = sprintf("%4d/%02d/%02d %02d:%02d:%02d", $year+1900,
				$mon+1, $day, $hour, $min, $sek);
##    my $timestamp = "$year/$mon/$day $hour:$min:$sek";

#    $level = uc($level);

    push @{$self->{log}}, "$timestamp> $level: $message";

}

sub getLogMessages {
    my $self = shift;

    return $self->{log};
}

sub logLogMessages {
    my $self = shift;
    my ($logger) = @_;
    $self->{logger} = $logger;

    $logger->debug("##### Start of logging CONF logs #####");
    foreach my $line ( @{$self->{log}} ) {
	my ( $time, $type, $message ) = ( $line =~ /^([\/ :\d]+)> (\w+?): (.+)/);
	$logger->$type($message);
    }
    $logger->debug("##### End of logging CONF logs #####");
}

sub log {
    my $self = shift;
    my ($type, $message) = @_;

    my $logger = $self->{logger};
    if ($logger) {
        $logger->$type($message);
    } else {
        $self->storeLogMessage($type, "$message\n");
    }
}

sub printLogMessages {
    my $self = shift;
    my ( $praefix ) = @_;

    foreach my $line ( @{$self->{log}} ) {
###	print "$line\n";
	my ( $time,$type,$message ) = ( $line =~ /^([\/ :\d]+)> (\w+?): (.+)/);

	if ( $message) {
	    if ( $type =~ /(WARN|ERROR)/ ) {
		$message = "${praefix}$type: $message\n";
	    }
	    else {
		$message = "${praefix}$message\n";
	    }
	    print $message;
	}
    }
}


sub newConf {
    my $self = shift;
    my ( $conf_file, $oldconf, $merge, $progname, $etcdir,
	 $preferredlanguage, $is_taint, $default_lang, $switchlang ) = @_;

    $is_taint = 0 if ! $is_taint;
#    open FH, ">>/tmp/CONF.debug";
#    print FH "newConf(): loading $conf_file\n";
    if ( $conf_file =~/\$progname/ ) {
	$conf_file =~s/\$progname/$progname/;
	$conf_file =~s/\\//;
    }
#    print FH "newConf(): loading $conf_file\n";
#
#    print FH "newConf(): taintmode is: |$is_taint|\n";
#    close FH;
    my %config;
#    $switchlang = 0;
#    $oldconf = $self->{data};

    if ( ! $switchlang ) {
	$self->log('debug', "newConf(): switchlang is not set");
	my $conf = new Config::General( -file =>$conf_file,
					-LowerCaseNames => 0,
#				    -ExtendedAccess => 1,
					-AllowMultiOptions => 'yes',
					-MergeDuplicateOptions => $merge,
					-MergeDuplicateBlocks => $merge,
					-InterPolateVars => 1,
					-DefaultConfig => $oldconf,
					-ConfigPath => $etcdir,
					-IncludeAgain => 1,
					-AutoLaunder => $is_taint,
	    );

	%config = $conf->getall;

	$self->{data} = \%config;

        ## Handle Inheritance
        $self->handle_inheritance();
    }
    else {
	$self->log('debug', "newConf(): switchlang is set");
    }

    if (defined $self->{data}{multilanguage}) {

	my @availablelanguages = $self->get_available_languages();
	$self->log('debug', "newConf(): available languages: @availablelanguages");

	my @acceptLangs;

	if ($preferredlanguage) {
            $self->log('debug', "newConf(): Preferred language is: $preferredlanguage");
	    @acceptLangs = ( $preferredlanguage );
	} else {
            my $acceptLanguage =  $ENV{'HTTP_ACCEPT_LANGUAGE'};
            if ($acceptLanguage) {
                $self->log('debug', "newConf(): The Browser sent: $acceptLanguage");
                @acceptLangs = split ',', lc $acceptLanguage;
                map {$_ =~ s/^([^;]+);.*$/$1/} @acceptLangs;
            }
	}

	$self->log('debug', "newConf(): Looking for @acceptLangs");

	my $found = '';
	### look for match (i.e. accepting en-us or en)
	foreach my $lang ( @acceptLangs ) {
	    if ( grep { $lang eq $_ }  @availablelanguages ) {
		$found = $lang;
		$self->log('debug', "newConf(): Found exact match");
		last;
	    } elsif ( grep { $lang =~ /$_-\w+$/ } @availablelanguages )  {
		$found = $lang;
		$found =~ s/^([^-]+)-\w+/$1/;
		$self->log('debug', "newConf(): Found approximate match");
		last;
	    } else {
		$self->log('warn', "newConf(): Language '$lang' not found in '@availablelanguages'");
	    }
	}

	### set default
	if ( !$found ) {
	    if ($self->{data}{preferredlanguage}) {
		$found = $self->{data}{preferredlanguage};
	    } else {
		$found = ''; # i.e. no extension
	    }
	}

	my $currentLocale = $found;
	$self->log('debug', "newConf(): selected Locale: '$currentLocale'");


	if (defined $currentLocale) {
	    if ( $default_lang && $currentLocale eq $default_lang ) {
		$currentLocale = '';
	    }
	    else {
		$currentLocale = '_'.$currentLocale;
	    }

	    # Build the locale based file name, aaa.bbb.conf --> aaa.bbb_en.conf
	    my $lc_conf_file = $conf_file;
	    $lc_conf_file =~ s/\.([^.]+)$/$currentLocale.$1/;

	    $self->log('debug',
                       "newConf(): Name of the locale based config file is: $lc_conf_file");

	    # Add locale based parameters only if the file exists.
	    if (-e $lc_conf_file) {
		$self->log('debug',
                           "newConf(): will read existing config file $lc_conf_file");

##		$oldconf = $self->{data};
		my $lc_conf =
		    new Config::General( -file =>$lc_conf_file,
					 -LowerCaseNames => 0,
					 -AllowMultiOptions => 'yes',
					 -MergeDuplicateOptions => 'yes',
					 -MergeDuplicateBlocks => 'yes',
					 -InterPolateVars => 1,
#				 -DefaultConfig => \%config,
					 -DefaultConfig => $oldconf,
					 -ConfigPath => $etcdir,
					 -IncludeAgain => 1,
					 -AutoLaunder => $is_taint,

		    );
		$self->log('debug',
			   "newConf():  now will call getall");
		my %newconfig = $lc_conf->getall;

##              use Data::Dumper qw(Dumper);
##		$self->log('debug',
##                           "newConf(): config now is: ".Dumper(\%newconfig));

		$self->{data} = \%newconfig;

                ## Handle Inheritance
                $self->handle_inheritance();
	    } else {
		$self->log('warn',
                           "newConf(): Config file $lc_conf_file not found");
	    }
	}
    }
}

##################################################################
# Name: handle_inheritance
#
# Type:
#
# Description: Adds attributes of the super item to the the derived
#              items (keyword 'super')
#
# Input: ~
#
# Output: ~
#
# Created: TZ 2017-07-03
# Changes:
#
######################################################################
sub handle_inheritance {
    my $self = shift;

    foreach my $item ( keys %{$self->{data}} ) {
#        print "$item\n";
        my $itemdata = $self->{data}{$item};
        if ( ref($itemdata) eq 'HASH' ) {
            foreach my $name ( keys %{$itemdata} ) {
                my $namedata = $self->{data}{$item}{$name};
                if ( ref($namedata) eq 'HASH' && exists $namedata->{super} ) {
                    my $super = delete $namedata->{super};
#                    print "Super is: $super\n";

                    if ( exists $self->{data}{$item}{$super} && $super ne $item ) {
                        my $superdata = $self->{data}{$item}{$super};

                        if ( ref($superdata) eq 'HASH' ) {
                            my $olditem = delete $self->{data}{$item}{$name};
#                            print "Old item is: ".Dumper($olditem);

                            ## Copy super
                            foreach my $key ( keys %{$superdata} ) {
                                $self->{data}{$item}{$name}{$key} = $superdata->{$key};
                            }
                            ## Add own attributes
                            foreach my $key ( keys %{$olditem} ) {
                                $self->{data}{$item}{$name}{$key} = $olditem->{$key};
                            }
#                            my $newitem = $self->{data}{$item}{$name};
#                            print "New item is: ".Dumper($newitem);
                        }
                    }
                }
            }
        }
    }
}

sub formatManPage {

    my $self = shift;

    my ( $lang ) = @_;

    # Formating to a length of LINELEN
    # the following lines are fully configurable commandline
    # parameters definable in the config-files ...


#   my $getOptParam;
    my ( $optName, $usage, @musts, %usage, $synopsis );
    my ( $mustmay, $with, $synopsisstring, @manpage);

    my @summary;

    my $progname =  $self->{data}{progname};
    my $lenfactor= 0;
    my $text;

    $synopsis =  $progname;

#    my $langdefault = $self->{data}{options}{language}{default};
#    my @languages = split /,\s*/, $self->{data}{options}{language}{values};
#    print "possible languages are: ".
#	$self->{data}{options}{language}{values}."\n";
#    print "language is set to: ". $self->{data}{language} ."\n"
#	if $self->{data}{language};

#    print "FORMATTING MANPAGE IN LANGUAGE $lang\n";

    foreach $optName ( sort keys %{$self->{data}{options}} ) {

#        $getOptParam .= $self->{data}{options}{$optName}{key};
#        $getOptParam .= ':' if $self->{data}{options}{$optName}{arg};

#	print "working on option $optName\n";
	$text = $self->formatusage($optName, $lang);

	if ( $self->{data}{options}{$optName}{key} ) {
	    $synopsisstring  = "-".$self->{data}{options}{$optName}{key};
	    push @summary, $synopsisstring." (".$optName.")";

	    $synopsisstring .= " ".$optName
		if $self->{data}{options}{$optName}{arg};;
	    $usage{$self->{data}{options}{$optName}{key}} = $text;
	}

	if ( $self->{data}{options}{$optName}{must} ) {
	    $mustmay = "required";
	    push ( @musts, $self->{data}{options}{$optName}{key} )
	} else {
	    $mustmay = "optional";
	    $synopsisstring = "[ ".$synopsisstring." ]";
	}

	if ( length ($synopsis." ".$synopsisstring) - $lenfactor  > LINELEN ) {
	    $lenfactor += length($synopsis);
	    $synopsis .= "\n\t".$synopsisstring;
	} else {
	    $synopsis .= " ".$synopsisstring;
	}
    }


    my $value;
    my $key;
    my $keykey;
    my @additions;


    push @manpage, "\n\nNAME\n\t$progname - $self->{data}{progshortdescr}\n";
    push @manpage, "\nSYNOPSIS\n\t$synopsis\n";
    push @manpage, "\n".$gt->text("DESCRIPTION")."\n",
                   $self->formatconfarray('progdescription');

    push @manpage, $gt->text("OPTIONS")."\n";
    foreach $usage (sort { "\L$a" cmp "\L$b" } keys %usage) {
	push @manpage,$usage{$usage};
    }


    foreach $key (sort keys %{$self->{data}{additions}}) {
	foreach $keykey (sort keys %{$self->{data}{additions}{$key}}) {
	    push @additions, "\t".&myformat("\t", "$self->{data}{additions}{$key}{$keykey}");
	}
	push @manpage, uc($key)."\n@additions\n";
        @additions = ();
    }

    push @manpage, "SEE ALSO\n",$self->formatconfarray('seealso');
    push @manpage, "AUTHOR\n\t$self->{data}{author}{name}, $self->{data}{author}{org}, ".
                   "$self->{data}{author}{mail}\n";
    push @manpage,"\nVERSION\n\t$self->{data}{version}, $self->{data}{date}\n";
    push @manpage, "\nCOPYRIGHT\n",$self->formatconfarray('copyright');
    push @manpage, "BUGS\n",$self->formatconfarray('bugs');


    $self->{data}{'synopsis'} = $synopsis;
    $self->{data}{'usage'} = \%usage;
    $self->{data}{'manpage'} = \@manpage;
    $self->{data}{'summary'} = \@summary;

}


sub checkOpts {
   my $self = shift;

   my $getOptParam;

   foreach my $optName ( sort keys %{$self->{data}{options}} ) {
#       print "Option: $optName\n";
       $getOptParam .= $self->{data}{options}{$optName}{key};
       $getOptParam .= ':' if $self->{data}{options}{$optName}{arg};
   }
   $self->{data}{'getOptParam'} = $getOptParam;


   my $error = 0;
   my %seen;
   my $string = $self->{data}{getOptParam};

   $string =~ s/://g;
   while ( $string =~ /(.)/g ) {
       if ($seen{$1} ) {
	   print "#### Error in .sys-file: \n";
	   print "\tOption key $1 is specified more than once!\n";
	   exit(-1);
       } else {
	   $seen{$1} = 1;
       }
   }

}


sub formatconfarray {
    my $self = shift;
    my ( $token) = @_;
    my ($key, @additions);

    foreach $key (sort keys %{$self->{data}{$token}}) {
	push @additions, "\t".&myformat("\t", "$self->{data}{$token}{$key}\n");
    }

    return ( @additions);
}

sub loadParams {

    my $self = shift;
    my $optName;

    if ( $self->{data}{getOptParam} ) {
	foreach $optName ( sort keys %{$self->{data}{options}} ) {
	    if ( $self->{data}{options}{$optName}{key} &&
		 defined($self->{data}{getopts}{$self->{data}{options}{$optName}{key}}) ) {
		$self->{data}{$optName} =
		    $self->{data}{getopts}{$self->{data}{options}{$optName}{key}};
	    }

	}

    }

}

sub checkAllValues {

    my $self = shift;

    my ( $optName, @values);
    my $abortProgram = 0;

    foreach $optName ( sort keys %{$self->{data}{options}} ) {
	if ( $self->{data}{$optName} &&
	     $self->{data}{options}{$optName}{values} ) {
	    $abortProgram += $self->checkValues($optName);
	}

    }
    return ($abortProgram);
}

sub checkValues {
    my $self = shift;
    my ( $optName) = @_;
    my ( @values);
    my $value_is_ok = 0;
    my $abortProgram = 0;

    @values = split (/, /, $self->{data}{options}{$optName}{values} );
    foreach my $value (@values) {
	my $val = uc($value);
	if ( $val =~ /^(.*)\?(.*)\?(.*)$/ ) {
	    if ( uc($self->{data}{$optName}) =~ /^$1.?$2.?$3$/  ) {
		#print "*** Found matching value [$value] = [$self->{data}{$optName}]\n";
		$value_is_ok = 1;
		last;
	    }
	} elsif ( $val =~ /^(.*)\?(.*)$/ ) {
	    if ( uc($self->{data}{$optName}) =~ /^$1.?$2$/  ) {
		#print "*** Found matching value [$value] = [$self->{data}{$optName}]\n";
		$value_is_ok = 1;
		last;
	    }
	} elsif ( $val  =~ /^(.*)\*(.*)$/ ) {
	    if ( uc($self->{data}{$optName})  =~ /^$1.*$2$/   ) {
		$value_is_ok = 1;
		last;
	    }
	} elsif ( $val eq uc($self->{data}{$optName})) {
	    $value_is_ok = 1;
	    last;
	}
	#print "Allowed Value: $value\n";
    }
    if (! $value_is_ok) {
	$self->printError($optName,
			  "Wrong value \"$self->{data}{$optName}\" in",
			  "The only currently allowed values are: ".
			  join (', ', @values) );

	$abortProgram += 1;
    }

    return ($abortProgram);
}


sub checkDependencies {

    my $self = shift;

    my $missingmust = 0;

    $missingmust = $self->checkMusts ();

    $missingmust += $self->checkRequirements ();

    return($missingmust);
}

sub checkArgType {
    my $self = shift;
    my @optarray = @_;

    my $arg_errors = 0;
#    my ($optName, $argtype, $maintype, $subtype, @additions);
    my ($optName, $argtype );

    my $old_errors;
    if ( $optarray[0] =~ /^\d+$/ ) {
	$old_errors = shift @optarray;
    }

    if ( ! @optarray ) {
	@optarray = sort keys %{$self->{data}{options}};
    }

    foreach $optName ( @optarray ) {
#	print "Option: $optName: $self->{data}{options}{$optName}{argtype}\n";
	if ( $self->{data}{$optName} && $self->{data}{options}{$optName}{argtype} ) {
	    foreach $argtype ( split (/;/, $self->{data}{options}{$optName}{argtype} )) {
#		print "checking argtype $argtype in option $optName\n";
		if ( ($old_errors || $arg_errors) && $argtype =~ /^ask_/ ) {
#		    print "#skipping ask $argtype because there is an error anyway\n";
		}
		else {
		    $arg_errors = $self->checkSingleArgType($optName, $argtype);
		}
	    }
	}
    }

    return ($arg_errors);

}


sub checkSingleArgType {
    my $self = shift;
    my ($optName, $argtype ) = @_;

    my $arg_error= 0;

    my ( $maintype, $subtype, @additions ) =  split (/_/, $argtype);
    if (! $subtype) {$subtype = "no"; }
#		print "Found argtype statement for option $optName\n";
#		print "maintype: [$maintype] subtype: [$subtype] additions: @additions\n";

    if ( $maintype =~ /^filename/ ) {
	$arg_error += $self->checkFiles ($optName, $subtype,
					 @additions);
    }
    elsif ( $maintype =~ /^uri/ ) {
	$arg_error += $self->checkUri ($optName, $subtype,
				       @additions);
    }
    elsif ( $maintype =~ /^checkfunctions/ ) {
	$arg_error += $self->checkFunctions ($optName, $subtype,
					     $self->{data}{$optName});
    }
    elsif ( $maintype =~ /^ask/ ) {
	$arg_error += $self->askParameter ($optName, $subtype,
					   @additions);
    }
    elsif ( $maintype =~ /^list/ ) {
	$arg_error += $self->checkList ($optName, $subtype,
					$self->{data}{$optName});
    }
    elsif ( $maintype =~ /^char/ ) {
	$arg_error += $self->checkChar ($optName, $subtype,
					$self->{data}{$optName});
    }
    elsif ( $maintype =~ /^int/ ) {
	$arg_error += $self->checkInt ($optName, $subtype,
				       $self->{data}{$optName});
    }
    elsif ( $maintype =~ /^token/ ) {
	$arg_error += $self->checkTokens($optName, $subtype,
					 $self->{data}{$optName});
    }
    elsif ( $maintype =~ /^skalar/ ) {
	if ( $subtype eq 'anyof' ) {
	    $arg_error += $self->checkAnyOf ($optName, $subtype,
					     @additions);

	}

	# nothing to check here ???
	;
    }
    else {
	print "#### Error in .sys-file: \n";
	print "\targtype ($maintype) of $optName unknown!\n";
	exit(-1);
    }
    return ( $arg_error );

}

sub checkList {
    my $self = shift;

    my ( $optName, $list_spec, $list) = @_;;

    my $list_error = 0;
    my @specs = split (/-/, $list_spec);

    my $sep = $self->{data}{separator};

    # Specs specify elements of a list with following syntax:
    # <char1><char2> where
    # <char1> specifies if field may be empty (=optional): 'o' or not (=mandatory): 'm'
    # <char2> specifies the fieldtype:
    #         s for string
    #         S for uppercase only string
    #         w for string containing one word
    #         c for character
    #         C for uppercase character
    #         d for decimal
    #         b for boolean (1 or 0)
    #         ##f for float
    #         ##F for Filter with format ( NOT attr == value && attr == value etc.)

##    print "Separator is: $self->{data}{separator}\n";

    my @list = $self->carefulSplit ($self->{data}{separator}, $list);

    my @listerrors;
    my $char;

    if ($#specs != $#list) {
#	print "specs: ".join('|', @specs)."\n";
#	print "list:  ".join('|', @list)."\n";
#	exit();
	push @listerrors,
	    "* ".($#list+1)." instead of ".($#specs+1).
	    " \"$sep\"-separated list elements";
    }

    for (my $i =0; $i <= $#specs; $i++ ) {
	if ($specs[$i] =~ /^m/ && ! $list[$i] ) {
	    push @listerrors, &addError($i, $list[$i],
					"value missing although mandatory");
	}
	$char = substr($specs[$i],1,1);
##	my $foundchar = 0;
	if ( $list[$i] ) {
	    if ($char eq 'c') {
		if ( $list[$i] !~ /^.$/) {
		    push @listerrors, &addError($i, $list[$i],
						"only one char expected");
		}
	    }
	    elsif ($char eq 'C' ) {
		if ($list[$i] !~ /^\u\w$/) {
		    push @listerrors, &addError($i, $list[$i],
					    "one upper case char expected");
		}
	    }
	    elsif ($char eq 'w' ) {
		if ($list[$i] !~ /^[\w\#:,-_*]+$/) {
		    push @listerrors, &addError($i, $list[$i],
						"word expected");
		}
	    }
	    elsif ($char eq 's' ) {
		my $stringchars = '\w\s,;:@_=|.$&+*~#!?"<>%/`(){}'
		    .'\\\[\]\'\^\-';

		if ( $list[$i] =~  /([^$stringchars]+)/ ) {
		    my $wrongchar= $1;
		    my $before = $`;
		    my $len = length($before);
		    if ( $len > 10 ) {
			$before = '...'.substr($before, $len-10);
		    }
		    push @listerrors, &addError($i, $list[$i],
				"string expected: $before->$wrongchar<-");
		}
	    }
	    elsif ($char eq 'W') {
		if ($list[$i] !~ /^[A-Z0-9_]+$/) {
		    push @listerrors, &addError($i, $list[$i],
						"upper case word expected");
		}
	    }
	    elsif ($char eq 'd') {
		if ( $list[$i] !~ /^\d+$/) {
		    push @listerrors, &addError($i, $list[$i],
						"only digits allowed");
		}
	    }
	    elsif ($char eq 'b' ) {
		if ( $list[$i] !~ /^[01]+$/) {
		    push @listerrors, &addError($i, $list[$i],
						"only 1 or 0 allowed");
		}
	    }
	    else {
		push @listerrors, &addError($i, $list[$i],
				  "unknown list specificator $char found");
	    }

	}

    }

    if ($#listerrors > 0) {
	my $heading;
	if ( lc($optName) =~ /file/ ) {
	    $heading = "Wrongly formatted line ".
		"(file $self->{data}{$optName}) in";
	} else {
	    $heading = "Wrong value for";
	}
	$self->printError($optName,$heading,
			  "found following ".($#listerrors+1)." suberrors:\n".
			  join ("\n", @listerrors) );
	$list_error = 1;
    }

    return ($list_error);
}


sub addError {

    my ($number, $value, $errtext) = @_;

    $number++;
    if (! $value ) { $value="";}
    my $string = "* in $number. element of list (\"$value\"):\n\t$errtext";
    return($string);
}


## carefull split that understands escaping
sub carefulSplit {
    my $self = shift;
    my ( $sep, $string, $quote ) = @_;

    my @array;
#    print "carefullSplit: sep: [$sep], quote: [$quote], string: $string\n";
    my $is_to_be_reversed = ( $string =~ s/\\$sep/\376/g);
#    print "OK is_to_be_reversed = $is_to_be_reversed\n";
### This was terrible wrong don't try again:
#    $is_to_be_reversed += ( $string =~ s/\"(.*?)$sep(.*?)\"/$1\376$2/g );

    if ($quote) {
	my $count =0;
	while ( $string =~ /$quote(.+?)$quote/ ) {
	    my $temp = $1;
	    my $before = $`;
	    my $after = $';
# ' #just for highlighting
	    $is_to_be_reversed += ( $temp =~ s/$sep/\376/g);
	    $string = "$before\377$temp\377$after";
#	    print "### string: [$string]\n";
	}

	$string =~ s/\377/$quote/g;

    }
    @array = split (/$sep/, $string, -1);

    if ($is_to_be_reversed) {
	foreach my $element (@array) {
	    $is_to_be_reversed -= ( $element =~ s/\376/$sep/g );
	    if (! $is_to_be_reversed) {last;}
	}
    }

    return (@array);


}


sub checkChar {
    my $self = shift;

    my ( $optName, $mode, $string) = @_;;

    my $error = 0;

    if ( $string && length($string) != 1 ) {
	$self->printError($optName,
			  "Wrong value (\"$self->{data}{$optName}\") for",
			  "Only a single character allowed here!");
	$error=1;
    }

    return ($error);

}

sub checkUri {
    my $self = shift;

    my ( $opt_name, $mode, @additions) = @_;

    my $error = 0;
    my $string = $self->{data}{$opt_name};
#    print "checking uri $string mode: $mode, additions: @additions\n";
    if ( $string ) {
	if ( $mode eq 'oneof' ) {
	    my $is_ok = 0;
	    foreach my $schema (@additions) {
		if ( $string =~ /^$schema/ ) {
		    $is_ok = 1;
		}
	    }
	    if ( ! $is_ok) {
		$self->printError($opt_name,
				  "Wrong value (\"$self->{data}{$opt_name}\") for",
				  "Only following Uri types are allowed here:".
				  "\n@additions");
		$error = 1;
	    }
	}
	else {
	    if ( $string =~ /^file:\s*(.+)/ ) {
		shift @additions;
#		print "starting checkfile for option $opt_name with mode exist and additions: @additions\n";
		$error += $self->checkFiles ($opt_name, "exist", @additions);
		#   print "checked filename $1\n";
	    } elsif ( $string =~ m@^ldaps://\s*(.+)@ ) {
		;
	    } elsif ( $string =~ m@^ldap://\s*(.+)@ ) {
		;
	    } elsif ( $string =~ /^dbi:\s*(.+)/i ) {
		;
	    } elsif ( $string =~ /^vhost:\s*(.+)/i ) {
		;
	    } else {
#		print "String: $string\n";
		if ( ! $error ) {
		    $self->printError($opt_name,
				      "Wrong value (\"$self->{data}{$opt_name}\")"
				      ." for",
				      "Only Uri types dbi:, file: and ldap:// "
				      ."are supported!");
		}
		$error=1;
	    }
	}
    }

    return ($error);

}


sub checkInt {
    my $self = shift;

    my ( $optName, $mode, $string) = @_;;

    my $error = 0;

    if ( $string && $string !~ /^\d+$/ ) {
	$self->printError($optName,
			  "Wrong value \"$self->{data}{$optName}\" in",
			  "Only a number is allowed here!");
	$error = 1;
    }

    if ( $mode && $mode ne "no" ) {

	my ($min, $max ) = split (/-/, $mode);

	if ( $string < $min || $string > $max ) {

	    $self->printError($optName,
			      "Wrong value \"$self->{data}{$optName}\" in",
			      "number must be between $min and $max!");
	    $error = 1;
	}

    }


    return ($error);
}

sub checkTokens {
    my $self = shift;

    my ($opt_name, $subtype, $string) = @_;

    $self->storeLogMessage('debug',
			   "checkTokens() with $opt_name, $subtype, $string");

    if ( $string ) {
	my @tmparray = split ( /;/, $string );
	foreach my $temp ( @tmparray ) {
	    my ($key, $value ) = split (/=/, $temp, 2);
	    $key =~ s/^#//;
	    $self->{data}{$subtype}{$key}=$value;
	    $self->storeLogMessage('debug', "setting $subtype/$key to $value");
    }
}




}


sub checkAnyOf {
    my $self = shift;

    my ($opt_name, $subtype, @allowedvalues) = @_;

    my @values = split /_/ , $self->{data}{$opt_name};

#    print "checking any of: @values\nallowed are  @allowedvalues\n";
    my $error = 0;
    foreach my $val ( @values ) {
	if (! grep /^$val$/, @allowedvalues )  {
	    $self->printError($opt_name,
			      "Wrong value (\"$self->{data}{$opt_name}\")",
			      " feature $val is unknown");
	    $error ++;
	}
    }

    return ($error);
}


sub checkFiles {
    my $self = shift;

    my ( $opt_name, $filemode, @additions) = @_;

##    print "checkFiles(): optname: $opt_name, mode: $filemode, else: |@additions|\n";

    my $filename = $self->{data}{$opt_name};

    $filename =~ s/^file://;

    if ( $additions[0] && $additions[0] eq 'subdir' ) {
	$filename = $self->getPathRight($filename, $self->{data}{progpath},
					$additions[1] ? "$additions[1]/" : '');
    }

    if ( $filename =~ m|^([\w./-]+)$| ) {
	$filename = $1;
    }
    else {
	die "\n####### Filename $filename contains invalid characters\n";
    }

    $self->{data}{$opt_name} = $filename;



    my $file_error = 0;
    my $filetype = 'file';
    $filetype = 'directory' if ( @additions && $additions[0] eq 'dir' );


#    print "checkfiles mode: $filemode, type: $filetype, additions: @additions\n";
    if ( $filemode eq "add" ) {
	;
    } elsif ( $filemode eq 'exist' || $filemode eq 'ok' ) {

	if ( ! -e $filename ) {
	    if ( $filemode eq 'exist' ) {
		$self->printError($opt_name,
				  "Wrong value (\"$self->{data}{$opt_name}\")",
				  " for this $filetype has to exist!");
	    }
	    else {
		$self->printError($opt_name,
				  "Thus cannot check @additions in", " ");
	    }

	    $file_error = 1;
	}

	if ( $filetype eq 'directory' ) {
	    if ( -e $filename && ! -d $filename ) {
		$self->printError($opt_name,
				  "Wrong value (\"$self->{data}{$opt_name}\")",
				  " for this has to be a directory!");
		$file_error++;
	    }

	}

	if (! $file_error &&  @additions) {
#-	    print "checking Fileformat in option $opt_name\n";
	    $file_error += $self->checkFileFormat ($opt_name, @additions);
	}

    } elsif ( $filemode eq "create" ) {
	if ( $filetype eq "directory" ) {
	    if ( ! -d $filename ) {
		print "creating dir $filename\n";
		mkdir $filename;
	    }
	}
 	elsif ( $additions[0] && $additions[0] eq 'bak' ) {
	    if ( -e $filename ) {
		my $bakfile = $filename.'bak'.$data->fileTimeStamp();
	        move($filename, $bakfile);
	    }
	}
	else {
	    if ($additions[0] && $additions[0] eq 'preserve' ) {
		if ( -e $filename ) {
		    my $bakfile = $filename.'-preserved';
		    copy($filename, $bakfile);
		}
	    }

	    if ( $self->{data}{force} ) {
		unlink $filename;
	    }
	    if ( -e $filename ) {
		$self->printError($opt_name,
			     "Wrong value (\"$self->{data}{$opt_name}\") for",
			     "this output file must not already exist!");
		$file_error = 1;
	    }
	}
    }

    return($file_error);
}



sub checkFileFormat {

    my $self = shift;
    my ($opt_name, @additions) = @_;
    my $format_error =0;

    my $max_lines = $self->{data}{testmaxlines};
    my $fp;
    my $i;
    my $is_list=0;
    my @list;
    my ($min, $max);

    #    print "Checking file content of $opt_name: [@additions] \n";

    if (  $max_lines && ($additions[0] eq "checkfunctions" ||
			 $additions[0] eq "format" )) {

    #	print "Checking file format of $self->{data}{$opt_name}
    #         (first $max_lines lines)\n";

	open $fp, "<$self->{data}{$opt_name}" ||
	    die "couldn't open file  $self->{data}{$opt_name} for read\n";
	$i =0;

	my $last_error=0;
	while (<$fp>) {
	    chomp;
	    if (/^\#/ || /^\s*$/ ) {next;}

	    if ( $additions[0] eq "format" ) {
		$last_error += $self->checkList($opt_name, $additions[2], $_);
	    } else {
		$last_error += $self->checkFunctions($opt_name, $additions[2], $_);
	    }
	    if ($last_error ) {
		$format_error += $last_error;
		print "\tline ".($i+1).": ".substr($_,0,60)." ...\n\n";
		$last_error = 0;
	    }
	    if ($i > $max_lines) {
		last;
	    }
	    $i++;
	}

    }

    return ($format_error);
}

sub checkNotyet {
    my $self = shift;

    my $optName;
    my $notyet;

    foreach $optName ( sort keys %{$self->{data}{options}} ) {
	if ( $self->{data}{options}{$optName}{notyet} ) {
	    if ( $self->{data}{$optName} ) {
		my $key = $self->{data}{options}{$optName}{key};
		print "   * Feature $optName (-$key) is not yet implemented\n";
		$notyet++;
	    }
	}
    }

    return($notyet);

}

sub checkMusts {
    my $self = shift;

    my $missingmust = 0;
    my ( $optName);

    foreach $optName ( sort keys %{$self->{data}{options}} ) {
	my $must = $self->{data}{options}{$optName}{must};
	my $ok = 0;
	if ( $must ) {
	    if ( ! $self->{data}{$optName} ) {
		if ( $must =~ /OR / ) {
		    my $musts = $must;
		    $ok = 0;
		    my @options;

		    while ( $musts =~ s/OR (\w+)// ) {
			push @options, $1;
			if (  $self->{data}{$1} ) {
			    $ok = 1;
			}
		    }

		    if ( $musts ) {
			$must = $musts;
		    }
		    elsif ( ! $ok) {
			my $optstring;
			if ( @options > 1 ) {
			   $optstring = "one of the options ";
			}
			else {
			   $optstring = "option ";
			}

			$self->printError($optName, "Missing alternative of",
					  "if this option is not set ".
					  "$optstring @options must be set");
			$missingmust = 1;
		    }
		}

		if ( ! $ok ) {
#		    print "must is nun: |$must|\n";
		    if ( $must =~ /^ASK/ ) {
			my ($dummy, @additions) = split /_/, $must;
			my $lang =  $self->getOptValue('language');
			my $name_lang = "name_$lang";
			my $targetname =
			    $self->{data}{options}{$optName}{$name_lang} ?
			    $self->{data}{options}{$optName}{$name_lang} :
			    $self->{data}{options}{$optName}{name};
#			print "additions0: ".$additions[0]."\n";
			if ( $additions[0] =~ /^ARGTYPE ?/) {
			    $additions[0] =
				$self->{data}{options}{$optName}{argtype};
#			    print "additions: ".$self->{data}{options}{$optName}{argtype}."\n";
			}

			$missingmust += $self->askParameter ($optName, $targetname,
							     @additions);

		    } else {
			$self->printError($optName, "Missing",
					  "This option was neither defined in the".
					  " configuration file (token $optName)".
					  " nor as comand line flag (-".
					  $self->{data}{options}{$optName}{key}.
					  ")");

			$missingmust = 1;
		    }
		}
	    }
	}
    }
    return ( $missingmust);
}


sub askParameter {
     my $self = shift;
     my ( $optName, $target, $mode ) = @_;

#     print "askparameter: opt: |$optName|, target: |$target|, mode: |$mode|\n";
     my $args = '';
     my $input;
     my $errors = 0;
     my $isCheck = 0;

     if ( $mode eq 'p' ) {
	 $input = prompt("   Please type in $target: ",  -echo=>'*' );
     }
     elsif ($mode =~ /^uri/ ) {
	 $input = $self->askUriParameter($optName, $target, $mode);
	 $isCheck = 1;
     }
     else {
	 $input = prompt("   Please type in $target: ");
	 $isCheck = 1;
     }

#??? was soll das:
     $self->{data}{$target}= $input;

     $self->{data}{$optName}= $input;
     $errors = $self->checkSingleArgType($optName, $mode)
	 if $isCheck;

     return ($errors);
}

sub askUriParameter {
    my $self = shift;
    my ( $optName, $target, $mode ) = @_;

    my $args = '';
    my $input;

    my ( $dummy, $type) = split /_/, $mode;

    my $prompt;
    my $returnstring = '';
    my @allowed;
    my $default;

    if ( $self->{data}{uritypes}{$type} ) {
	print "Collecting information for "
	    .$self->{data}{uritypes}{$type}{name}."\n";

	foreach my $part ( sort keys %{$self->{data}{uritypes}{$type}{parts}} ) {
	    $prompt = "   Please type in ".
		$self->{data}{uritypes}{$type}{parts}{$part}{prompt};

	    if ( $self->{data}{uritypes}{$type}{parts}{$part}{choose} ) {
		@allowed = split /_/,
		$self->{data}{uritypes}{$type}{parts}{$part}{choose};
		$prompt .= " (allowed values: @allowed)";
	    }
	    $default = $self->{data}{uritypes}{$type}{parts}{$part}{default};

	    $prompt .= $default ? " [$default]: " : " []: ";
	    $input = prompt($prompt);
	    if ( ( $input ne "" || $default ) &&
		 $self->{data}{uritypes}{$type}{parts}{$part}{before} ) {
		$returnstring .= $self->{data}{uritypes}{$type}{parts}{$part}{before};
	    }

	    if ( $input eq ""  && $default ) {
		$returnstring .= $default;
	    }
	    elsif ( $input eq "" &&
		    $self->{data}{uritypes}{$type}{parts}{$part}{must} ) {
		while ( $input eq "" ) {
		    print "### You have to specify " .
			$self->{data}{uritypes}{$type}{parts}{$part}{prompt} .
			"!!!\n";
		    $input = prompt($prompt);

		}
		$returnstring .= $input;
	    }
	    else {
		$returnstring .= $input;
	    }

	    if ( $self->{data}{uritypes}{$type}{parts}{$part}{after} ) {
		$returnstring .= $self->{data}{uritypes}{$type}{parts}{$part}{after};

	    }

	}
    }
    else {
	print "#### Error in .sys-file: \n";
	print "\tNo definition of uritype $type!\n";
	exit(-1);

    }

    print "returning uri: \n\t$returnstring\n"
	if $self->is_verbose();
    return ( $returnstring );
}


sub checkFunctions {
     my $self = shift;

     my ( $optName, $list_spec, $list) = @_;
     my $errors =0;
#     print "checking Functions in option $optName value: [$list]\n";
#     exit();
     my $function;
     while ( $list =~ s/&{2,3}(.*?)\(// ) {
	 $function = $1;
#	 use Data::Dumper;
#	 print '|'.Dumper($self->{data}{functions}).'|\n';
#	 exit();
	 if ( ! $self->{data}{functions}{$function} ) {
	     if ( $function !~ /-to-/ && $function !~ /^x/ ) {
		 $self->printError($optName,
			   "nonexisting function |$function| referenced in",
			   "only implemented functions can be referenced!");
		 $errors++;
	     }
	 }

     }
     return ($errors);
}


sub checkRequirements {
    my $self = shift;

    my $reqnotfulfilled = 0;
    my ($optName, $requirement, $reqtype, $reqcond);

    foreach $optName ( sort keys %{$self->{data}{options}} ) {

	if ( $self->{data}{$optName} && $self->{data}{options}{$optName}{requires} ) {
	    foreach $requirement ( split (/;/, $self->{data}{options}{$optName}{requires} )) {


		if ( $requirement =~ / OR / ) {
		    $reqnotfulfilled += $self->checkOrReq ($optName,
							   $requirement
							   );


		} else {

		    $reqnotfulfilled += $self->checkSingleReq($optName,
							      $requirement,
							      1 );
		}
	    }
	}
    }

    return ($reqnotfulfilled);

}


sub checkSingleReq {
    my $self = shift;
    my ($optName, $requirement, $is_printerr) = @_;

    my $reqnotfulfilled=0;
    my ( $reqtype, $reqcond ) =  split (/ = /, $requirement);
    #    print "Found requirement statement [$requirement]\n";
    #    print "type: [$reqtype] condition: [$reqcond]\n";

    if ( $reqtype =~ /NOT/ ) {
	$reqnotfulfilled += $self->checkNOTReq ($optName,
						$reqtype,
						$reqcond,
						$is_printerr);
    } elsif ( $reqtype =~ /EXIST/ ) {
	$reqnotfulfilled += $self->checkEXISTReq ($optName,
						  $reqtype,
						  $reqcond,
						  $is_printerr);
    } elsif ( $reqtype =~ /if value/ ) {
	$reqnotfulfilled += $self->checkIfValueReq ($optName,
						    $reqtype,
						    $reqcond,
						    $is_printerr);
    } else {
	$reqnotfulfilled += $self->checkSimpleReq ($optName,
						   $reqtype,
						   $reqcond,
						   $is_printerr);
    }

    return ($reqnotfulfilled);
}

sub describeOrReq {
    my $self = shift;
    my ($requirement) = @_;

    my $text = '';

    my $or = $gt->text("or");

#    print "describeOrReq() got: |$requirement|\n";
    foreach my $req (split ( / OR /, $requirement )) {
	$text .= $self->describeSingleReq($req)
	    .' '.$or.' ';
    }

    $text =~ s/ $or $//;

    return ($text);
}

sub describeSingleReq {
    my $self = shift;
    my ($requirement) = @_;

    my $text = '';
    $requirement =~ s/^\s+//;
#    print "describeSingleReq() got: |$requirement|\n";


    my ( $reqtype, $reqcond ) =  split (/ = /, $requirement);

##    $reqtype =~ s/^\s*//;
#print "single req is: |$reqtype = $reqcond| from |$requirement|\n";
    if ( $reqtype =~ /^EXIST/ ) {
	$text = $gt->text("option").' '.$reqcond.' '
	    . $gt->text("must be set");
    }
    elsif ( $reqtype eq 'NOT' ) {
	$text = $gt->text("option").' '.$reqcond.' '
	    . $gt->text("must not be set");
    }
    elsif ( $reqtype eq 'if value' ) {
	my ( $val, $cond ) = split /: /, $reqcond;
	$text = $gt->text("if value is").' "'.$val.'" '.$gt->text("then");
	if ( $cond =~ / \| / ) {
	    my ( $one, $two ) = split / \| /, $cond;
	    $text .= ' '.$gt->text("either").' '.$one.' '
		.$gt->text("or").' '.$two.' '.$gt->text("must be set");
	}
	else {
	   $text .= ' '.$cond.' '.$gt->text("must be set");
	}
    }
    else {
	$text .= $gt->text("option").' '.$reqtype.' '
	    .$gt->text("has to contain the value").' "'.$reqcond.'"';
    }
#    print "returning text: $text\n";
    return $text;
}


sub checkOrReq {
    my $self = shift;
    my ( $optname, $requirement) = @_;
    my $reqnotfulfilled = 0;
    my ( $reqtype, $reqcond );
    my $is_fulfilled = 0;
    my @conds;

    my @reqs = split ( / OR /, $requirement );

    foreach my $req (@reqs) {
#        print "checking requirement $req\n";

	( $reqtype, $reqcond ) =  split (/ = /, $req);
#	$reqnotfulfilled = $self->checkSimpleReq ($optname,
#						  $reqtype,
#						  $reqcond,
#						  0);

	$reqnotfulfilled = $self->checkSingleReq ( $optname, $req, 0);
	push @conds, $reqcond;

	if ( ! $reqnotfulfilled ) {
	    $is_fulfilled =1;
	}
    }

    if (! $is_fulfilled ) {

	$self->printError($optname,
			  "unfulfilled requirement of",
			  "This option requires that option \"".
			  $reqtype."\" has one of the following values:".
			  " @conds ".
			  "either defined in the configuration file ".
			  "(\"$reqtype = $reqcond\") or as comand line".
			  " flag (\"-".$self->{data}{options}{$optname}{key}.
			  " $reqcond\")\n");
	$reqnotfulfilled = 1;
    } else {
	$reqnotfulfilled = 0;
    }

    return ($reqnotfulfilled)
}

sub checkSimpleReq {
    my $self = shift;
    my ( $optName, $reqtype, $reqcond, $is_printerror) = @_;

    my $reqnotfulfilled = 0;

    $reqtype =~ s/^ //;
    $reqtype =~ s/ $//;

    if ( ! $self->{data}{options}{$reqtype}{key} ) {
	print "#### Error in .sys-file: \n";
	print "\trequirement of $optName refers to a nonexisting option [$reqtype]!\n";
	exit(-1);
    }

#    print "optname $optName, condition: $reqcond type $reqtype exists: [$self->{data}{$reqtype}]\n";
    if (  $self->{data}{$reqtype} && $self->{data}{$reqtype} !~ /$reqcond/ ) {
	if ( $is_printerror ) {
	    $self->printError($optName,
			      "unfulfilled requirement of",
			      "This option requires that option \"".
			      $reqtype."\" has value \"$reqcond\", ".
			      "either defined in the configuration file ".
			      "(\"$reqtype = $reqcond\") or as comand line".
			      " flag (\"-".$self->{data}{options}{$optName}{key}.
			      " $reqcond\")\n");
	}
	$reqnotfulfilled = 1;
    }
    return ($reqnotfulfilled);
}

sub checkNOTReq {
    my $self = shift;
    my ( $optName, $reqtype, $reqcond, $is_printerror) = @_;

    my $reqnotfulfilled = 0;

#    print "checkNOTReq option $optName, type $reqtype, cond $reqcond\n";

    if ( ! $self->{data}{options}{$reqcond}{key} ) {
	print "#### Error in .sys-file: \n";
	print "\trequirement of $optName refers to a nonexisting option $reqcond!\n";
	exit(-1);
    }

    if ( $self->{data}{$reqcond} ) {
	if ( $is_printerror ) {
	    $self->printError($optName, "Unfulfilled requirement of",
			      "If this option is set ".
			      "the option $reqcond must not be set!");
	}
	$reqnotfulfilled = 1;
    }

    return($reqnotfulfilled);
}


sub checkEXISTReq {
    my $self = shift;
    my ( $optName, $reqtype, $reqcond, $is_printerror) = @_;

    my $reqnotfulfilled = 0;

    if ( ! $self->{data}{options}{$reqcond}{key} ) {
	print "#### Error in .sys-file: \n";
	print "\trequirement of $optName refers to a nonexisting option $reqcond!\n";
	exit(-1);
    }

    if ( ! $self->{data}{$reqcond} ) {
	if ( $is_printerror ) {
	    $self->printError($optName, "Unfulfilled requirement of",
			      "If this option is set ".
			      "the option $reqcond must also be set!");
	}
	$reqnotfulfilled = 1;
    }

    return($reqnotfulfilled);
}


sub checkIfValueReq {

    my $self = shift;
    my ( $optName, $reqtype, $reqcond, $is_printerror ) = @_;

    my ( $reqval, @musts ) ;
    my $reqnotfulfilled = 0;
    my $is_or = 0;
    my $must;

    ($reqval, $reqcond) = split (/: /, $reqcond);

    if ( $reqcond =~ / \| / ) {
	$is_or = 1;
	@musts = split ( / \| /, $reqcond );
    } elsif ( $reqcond =~ / \& / ) {
	$is_or = 0;
	@musts = split ( / \& /, $reqcond );
    } else {
	$is_or = 1;
	$musts[0] = $reqcond;
    }

    my $isok = 0;
    foreach $must (@musts) {
	if ( ! $self->{data}{options}{$must}{key} ) {
	    print "#### Error in .sys-file: \n";
	    print "\trequirement of $optName refers to a nonexisting option $must!\n";
	    exit(-1);
	}
    }

    if ( $self->{data}{$optName} =~ /$reqval/ ) {
	foreach $must (@musts) {
	    if ( $self->{data}{$must} ) {
		$isok=1;
		if ( $is_or ) {
		    last;
		}
	    } else {
		$isok = 0;
		if ( ! $is_or ) {
		    last;
		}
	    }
	}
	if ( ! $isok ) {
	    $reqnotfulfilled = 1;
	    if ( $is_printerror ) {
		my $textbit;
		if ( $is_or ) {
		    $textbit = "one";
		} else {
		    $textbit = "all";
		}
		$self->printError($optName,"unfulfilled requirement of",
				  "If its value equals \"$reqval\", ".
				  "this option requires that $textbit of ".
				  "the following options has been set ".
				  "either defined in the configuration file ".
				  "or as comand line flag: ".
				  join (', ', @musts) );
	    }
	}
    }

    return ($reqnotfulfilled);
}

sub setConfDefaults {

    my $self = shift;
    my ( $progname ) = @_;
    my ( $optName );


    foreach $optName ( sort keys %{$self->{data}{options}} ) {
	my $default = $self->{data}{options}{$optName}{default};
	if ( !$self->{data}{$optName} && $default ) {
	    if ( $default =~ /\$progname/ ) {
		$default =~s/\$progname/$progname/g;
		$default =~s/\\//;
	    }
	    $self->{data}{$optName} = $default;
	}
    }

    if ( !  $self->{data}{separator} ) {
	$self->{data}{separator} = ',';
    }

}


sub getOptions {

    my $self = shift;

    my ( %opts );

    if ( $self->{data}{getOptParam} ) {
        if ( !getopts($self->{data}{getOptParam}, \%opts) ) {
            die "\n\tUsage:\n\t" . $self->{data}{synopsis}."\n\n";
        }
	$self->{data}{getopts} = \%opts;
    }

}


sub getOptValue {
    my $self = shift;
    my ( $optname ) = @_;

    my $value = "";
    my $key = $self->{data}{options}{$optname}{key};

    if ( $key) {
	if ( $self->{data}{getopts}{$key} ) {
	    $value = $self->{data}{getopts}{$key};
	}
    }
    return ($value);
}


sub printConf {

    my $self = shift;
    my ( $headline ) = @_;

    my ( $optName );

    print "$headline\n" if $headline;

    foreach $optName ( sort keys %{$self->{data}{options}} ) {
	if ( $self->{data}{$optName} ) {
	    print "\t$optName = $self->{data}{$optName}\n" ;
	} else {
	    print "\t$optName = [no value set]\n";
	}
    }

}

sub getConf {

    my $self = shift;
    my ( $headline ) = @_;

    my ( $optName );

    my $result = '';

    $result .= "$headline\n" if $headline;

    foreach $optName ( sort keys %{$self->{data}{options}} ) {
	if ( $self->{data}{$optName} ) {
	    $result .= "\t$optName = $self->{data}{$optName}\n" ;
	} else {
	    $result .= "\t$optName = [no value set]\n";
	}
    }

    return $result;
}

sub formatusage {
    my $self = shift;

    my ($optName, $lang) = @_;

    my $text;
    my $sep = "\t\t";
    my $initsep;


    if ( $self->{data}{options}{$optName}{key} ) {
	if ( $self->{data}{options}{$optName}{arg} ) {
	    $text = "\t-" . $self->{data}{options}{$optName}{key}." "
		. $optName."\n";
	    $initsep = "\t\t";
#	    . $optName."\n\t\t";
	} else {
	    $initsep = "\t\t";
	    $text =  "\t-" . $self->{data}{options}{$optName}{key}." ["
		. $optName."]\n";;
#."\t";
	}
    }

    my $desc_lang = "description_$lang";

    my $description = $self->{data}{options}{$optName}{$desc_lang} ?
	$self->{data}{options}{$optName}{$desc_lang} :
	$self->{data}{options}{$optName}{description};

    if ( $description) {
	$description =~ s/\\n/\n/g;
	$text .= $self->myuformat($initsep, $sep, $description."\n");
    }

    my $argtype = $self->{data}{options}{$optName}{argtype};
    if ( $argtype && $argtype =~ /^uri_/ ) {
	my ($uritype) = (  $argtype =~ /^uri_(.*)/ );

	foreach my $part ( sort keys %{$self->{data}{uritypes}{$uritype}{parts}} ) {
#	    print "uritype: $uritype part: $part\n";
#### partdescriptions ok???

	    my $partdescription =
		$self->{data}{uritypes}{$uritype}{parts}{$part}{$desc_lang} ?
		$self->{data}{uritypes}{$uritype}{parts}{$part}{$desc_lang} :
		$self->{data}{uritypes}{$uritype}{parts}{$part}{description};

	    if ( $partdescription ) {
		$text .= $self->myuformat($initsep, $sep, $partdescription."\n");
#		print "uritype: $uritype\n";
#		print "partdescription: $partdescription\n";
	    }
	}
    }


    if ( $self->{data}{options}{$optName}{experimental} ) {
	$text .= $self->myuformat($initsep, $sep,
				  $gt->text("BEWARE: This feature is ".
					    "experimental and not yet ".
					    "fully supported")."\n");
    }


    $initsep = "\t\t";

    if ( $self->{data}{options}{$optName}{values} ) {
	$text .= $self->myuformat($initsep, $sep,
			  $gt->text("Possible values are").": "
			  . $self->{data}{options}{$optName}{values}
			  . "\n");
    }
    elsif ( $self->{data}{options}{$optName}{argtype} ) {
	if ( $self->{data}{options}{$optName}{argtype} =~ /^filename_exist_dir/ ) {
	    $text .= $initsep.$gt->text("The directory has to exist").".\n";
	}
	elsif ( $self->{data}{options}{$optName}{argtype} =~ /^filename_exist/ ) {
	    $text .= $initsep.$gt->text("The file has to exist").".\n";
	}
	elsif ( $self->{data}{options}{$optName}{argtype} eq 'skalar' ) {
	    ;
	}
	else {
	    $text .= $initsep.$gt->text("The syntax will be checked").".\n";
	}
    }

    if ( $self->{data}{options}{$optName}{requires} ) {
	my $ismulti = 0;
	my $reqtext;
	my @reqs = split /;/,  $self->{data}{options}{$optName}{requires};
#	if ( $optName eq 'inform' ) {
#	    print "string: !$self->{data}{options}{$optName}{requires}!\n";
#	    print "\@rqs: !@reqs!\n";
#	    print join 'XXX', @reqs;
#	    print "\ncount: $#reqs\n";
#	    exit();
#	}

	if ( $#reqs > 0 ) {
	    $ismulti = 1;
	    $reqtext =
		$gt->text("The following condition(s) have to be fulfilled");
	}
	else {
	    $reqtext =
		$gt->text("The following condition has to be fulfilled");
	}
	$reqtext .= ': ';
#	print "ori: |$self->{data}{options}{$optName}{requires}|\n";
#	my $aswellas = $gt->text("as well as");
	my $aswellas = '';
	my $count = 1;
	foreach my $req ( @reqs) {
#	    print "single req: |$req|\n";
	    $reqtext .= "\n";
	    $reqtext .= "$count.) " if $ismulti;
	    if ( $req =~ / OR / ) {
		$reqtext .= $self->describeOrReq($req);
	    }
	    else {
		$reqtext .= $self->describeSingleReq($req);
	    }
	    $reqtext .= " $aswellas " if $ismulti;
	    $count++;
	}

	$reqtext =~ s/ $aswellas $//;

#	if ( $optName eq 'inputuri') { print "*** inputuri req: $reqtext\n";}
	$text .= $self->myuformat($initsep, $sep,"$reqtext\n");

    }

    if ( $self->{data}{options}{$optName}{must} ) {
	if ( $self->{data}{options}{$optName}{must} =~ /^OR (.+)/ ) {
	    my $oroption = $1;
	    if ( $oroption =~ / OR / ) {
		my $oroptiontext = $gt->text("This option or option");
		foreach ( split / OR /, $oroption ) {
		   $oroptiontext .= " $_ (-"
		                 . $self->{data}{options}{$_}{key}.")"
				 . " ".$gt->text("or")." "
				 .$gt->text("option")." ";
		}
		my $xx = $gt->text("or")." ". $gt->text("option");
		$oroptiontext =~ s/ $xx $/ /;
		$text .= $self->myuformat($initsep, $sep,
#					  $gt->text("This option or option")
#					  . ' ' . $oroptiontext
					  $oroptiontext
					  . $gt->text("MUST be set")."!\n");
	    }
	    else {
		my $orkey = $self->{data}{options}{$oroption}{key};
		$text .= $self->myuformat($initsep, $sep,
					  $gt->text("This option or option")
					  . " "
					  . $oroption . ' (-'.$orkey.') '
					  . $gt->text("MUST be set")."!\n");
	    }
	}
	elsif ( $self->{data}{options}{$optName}{must} =~ /^ASK/ ) {
	    $text .= $initsep.$gt->text("If not set this option will be prompted for").".\n";
	}
	else {
	    $text .= $initsep.$gt->text("This option MUST be set")."\n";
	}
    }

    if ( $self->{data}{options}{$optName}{default} ) {
	$text .= $self->myuformat($initsep, $sep,
			  $gt->text("Default is").': "'
			  . $self->{data}{options}{$optName}{default}
			  . "\"\n");
    }
    else {
	$text .= $initsep.$gt->text("No default")."\n";
    }

    $text .= "\n";

    if ( $text) {
	$text =~ s/\\n/\n\t\t/g;
#	$text =~ s/\\t/\t/g;
	$text =~ s/\\&/&/g;
	$text =~ s/\\#/#/g;
	$text =~ s/\\"/\"/g;
#"
    }

    return $text;

}


### should be moved to DAASIlib::Data
sub myformat {

    my ( $sep, $line ) = @_;
#    my @lines;
    my $wo;
    if ( ! $ISWEB ) {
	$Text::Wrap::columns = LINELEN - 5;
	$line = wrap('', $sep, $line);
    }
#    $line = autoformat $line, { left=>18, right=>70 };
    $line .= "\n";
    return ($line);

}


sub myuformat {
    my $self = shift;
    my ($initsep,  $sep, $line ) = @_;
    if ( ! $ISWEB ) {
	$Text::Wrap::columns = LINELEN - 5;
	$line = wrap($initsep, $sep, $line);
    }
#    $line = autoformat $line, { left=>18, right=>70 };
    $line .= "\n";
    return ($line);
}


### From LDIF.pm may be this can be modified to be used here

sub _wrap {
  my $len=$_[1];
  return $_[0] if length($_[0]) <= $len;
  use integer;
  my $l2 = $len-1;
  my $x = (length($_[0]) - $len) / $l2;
  my $extra = (length($_[0]) == ($l2 * $x + $len)) ? "" : "a*";
  join("\n ",unpack("a$len" . "a$l2" x $x . $extra,$_[0]));
}


#sub _wrap {
#    my $self = shift;
#    my $string=$_[0];
#    my $len=$_[1];
#    my $a=(length($string)-$len>0);
#    my $b=((length($string )-$len)/($len-1));
#    join("\n ",unpack("a78" x $a . "a77" x $b . "a*",$string));
#}

sub printError {
    my $self = shift;
    my ( $opt_name, $heading, $errtext) = @_;

    print " Error: \u$heading option \"$opt_name\":\n";
    if ($self->is_verbose() ) {
	print &myformat("\t",
			"\t($self->{data}{options}{$opt_name}{description}".
			")\n\n");
    }
    print &myformat("\t","\t$errtext\n");

}

sub addFunctionParameters {
    my ( $self) = shift;

    my $functionconf = $self->{data}{functions};

    my %locations;

    foreach my $function ( keys %{$functionconf} ) {
	my $i =1;
	my $para = "parameter$i";

	if ( $functionconf->{$function}{location} ) {
#    print "adding location ", $functionconf->{$function}{location},"\n";
	   $locations{$functionconf->{$function}{location}}++;
	}

	while ( $functionconf->{$function}{$para} ) {
	    if ( $functionconf->{$function}{$para} ne "string" ) {
		if ( $functionconf->{$function}{$para} eq 'etcdir') {
#		    print "storing $self->{data}{etcdir} to $para of function $function\n";
		    $functionconf->{$function}{$para} = $self->{data}{etcdir};
		}
		elsif (  $self->{data}{$functionconf->{$function}{$para}} ) {
		    $functionconf->{$function}{$para} =
			$self->{data}{$functionconf->{$function}{$para}};

		} else {
## since we have checkfunctions now, we don't need the following:
#		    print "Error: wrong functionparameter $para of \$[$function] in .sys file\n";
#		    print "looked for [$functionconf->{$function}{$para}]\n";
#		    print "##: [$conf->{data}{$functionconf->{$function}{$para}}]\n";
		}
	    }
	    $i++;
	    $para = "parameter$i";
	}

    }

    @{$self->{data}{functionlocations}} = keys %locations;


#    return ($functionconf);
}

sub getPathRight {
    my $self = shift;
    my ($fullfilename, $progpath, $subdir, $addtoetc) = @_;

#    $self->log('debug', "getPathRight(): filename: $fullfilename, progpath: $progpath, subdir: $subdir, addto: $addtoetc");

    my $dir;
    if ( $subdir && $subdir =~ m{^/} ) {
	$dir = $subdir;
    }
    elsif ( $subdir ) {
	$dir = "$progpath$subdir";
    }
    else {
	$dir = "$progpath";
    }

    if ( ! -e $fullfilename ) {
	my ($filename, $dirname) = fileparse($fullfilename);
#    $self->log('debug', "getPathRight(): ######## file $fullfilename does nor exist. ($filename, $dirname)");
	if ( $fullfilename =~ m{^\./(.+)} ) {
	    $fullfilename = "$progpath$1";
	    if ( ! -e $fullfilename ) {
		$fullfilename = "$dir$filename";
	    }
	}
	elsif ( $dirname =~ m{^/} ) {
	    ## just do nothing and let checkFiles report an error
	}
	else {
	    $fullfilename = "$dir$filename";
# print "thinking this is it: $fullfilename\n";
	}
    }
#    $self->log('debug', "getPathRight(): returning $fullfilename");

    return ($fullfilename);
}

##################################################################
# Name: get_available_languages
#
# Type:
#
# Description: Returns the list of configured languages
#
# Input: ~
#
# Output: Array of possible languages
#
# Created: TZ 2017-04-03
# Changes:
#
######################################################################
sub get_available_languages {
    my $self = shift;
    my @availablelanguages = ();

    if (defined $self->{data}{multilanguage}) {
	my $availablelanguagesconf = $self->{data}{availablelanguages} || '';
=for Configdoc
 availablelanguages

 Typ: general CP
 Ort: .sys oder equivalent (

 Beschreibung:
 can cope with HTTP_ACCEPT_LANGUAGE: 'fr-FR,en-US;q=0.8,de;q=0.5,en-US;q=0.3';
 in ldapportal.conf: introduced new cp availablelanguages, e.g. 'de;en';
 preferredlanguage is being honored. It is now no longer needed to create
 symlinks like fields_en-gb.conf pointing to fields_en.conf

 Abhängigkeiten: CP flag multilanguage

 Erlaubte Werte: -

 Default: -
=cut

	if (! $availablelanguagesconf ) {
	    if ($self->{data}{preferredlanguage}) {
		$availablelanguagesconf = $self->{data}{preferredlanguage};
	    }
	}

        $self->log('debug', "get_available_languages(): This application can offer the following languages: '$availablelanguagesconf'");

	@availablelanguages = split ';', lc $availablelanguagesconf;
    }

    return @availablelanguages;
}




1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

DAASIlib::CONF - Perl extension for configuration management

=head1 SYNOPSIS

  use DAASIlib::CONF;

  my $conf = new DAASIlib::CONF

  $conf->loadConfig;

  if ($conf->is_debug) {
   ...
  }

  if ($conf->{data}{myoption} = "XXX" ) {
   ...
  }

=head1 DESCRIPTION

Perl extension for configuration management based on Config::General and Getopt::Std
the basic idea is that by an internal configuration file you can specify all
configurable options. These options then can be configured by the user either
via an user config file or via command line parameters.

The module also creates a formated manpage from the internal configuration file.

The structure of the internal configuration file is
[text in square brackets should be replaced by your values]:


progname = [name of your Program]

version = [version number of your program]

date = [Date of that version]

<author>
   name  = [name of the program author]
   org= [organisation of the program author]
   mail = [email address of the program author]
</author>

<copyright>
par1 = [first paragraph of your copyright statement]
par2 = [second paragraph of your copyright statement]
[...]
</copyright>

progshortdescr = [short description of your program]

<progdescription>
par1 = [first paragraph of your long description of your program]
par2 = [second paragraph of your long description of your program]
[...]
</progdescription>

<seealso>
par1 = [first paragraph of your see also statement]
par2 = [second paragraph of your see also statement]
[...]
</seealso>

<bugs>
par1 = [first paragraph of your bugs statement]
par2 = [second paragraph of your bugs statement]
[...]
</bugs>

<additions [heading of your first additional manpage paragraph]>
line1 = [first formatted line of your first additional manpage paragraph]
line2 = [second formatted line of your first additional manpage paragraph]
line3 = " "
[...]
</additions>

<additions [heading of your second additional manpage paragraph]>
[...]
</additions>

<options debugmode>
        key = [flag you want to use for setting debugmode]
        must = [set this to 0 because option is optional]
        description = [Text describing this option, e.g. "sets debug mode to on"]
        arg = [set this to 0 because options needs no additional argument]
</options>

<options configfile>
        key = [flag you want to use for setting configfile]
        must = [set this to 0 because option is optional]
        description = [Text describing this option, e.g. "Name of the user config
                       file with absolute or relative path. Default:
                       ./etc/dbconnector.conf" Beware to prevent long lines use \
at the end of the line. Formatting will be donme automatically ]
        arg = [set this tp 1 because this option needs an argument]
        default = [set a default config file name, e.g. "./etc/dbconnector.conf"]
</options>

<options printhelp>
        key = [flag you want to use for printing the manopage, e.g. "h"]
        must = 0
        description = "prints out the manpage"
        arg = 0
</options>

#now you can define any other options for your program, using this XMLish structure

<options [your optionname used in your program]>
        key = [flag you want to use for this option]
        must = [does the user have to set this option via command line parameter? 0/1]
        description = [description of your option]
        arg = [does this option need an additional argument? 0/1]
</options>

The user config file should then have the following format:

   option = value

All options defined in the internal config file cann be set via the user config file.

e.g.:

inputfile = /home/myaccount/etc/myinputfile

it makes little sense to use the predefined options configfile, debugmode and printhelp
within the user config file.


=head2 EXPORT

None by default.

=head1 SEE ALSO

This Module works fine with DAASIlib::HTML, DAASIlib::LDAP

=head1 AUTHOR

Peter Gietz, <peter.gietz@daasi.de>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by DAASI International GmbH

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
