#######################################################################
# Copyright: DAASI International GmbH 2018. All rights reserved.
#
# This is Open Source Software
# License: Artistic License 2.0
#
# Author: Peter Gietz, DAASI International GmbH, www.daasi.de
# For questions please mail to peter.gietz@daasi.de
#######################################################################
package DAASIlib::Cert;

use strict;
use warnings;
use vars qw($VERSION);

use DAASIlib::LOG;
#use Log::Log4perl::Level;

my $log = new DAASIlib::LOG;
my $logger = Log::Log4perl->get_logger("main"); 

use MIME::Base64;
# qw (encode_base64);
#use Convert::ASN1 qw(:io :debug);
#use Convert::PEM;

our $VERSION = "0.1";
our $ISDEBUG=0;

sub new
{
    my $class;
    my $self;

    my ( $this , $param ) = @_;
    $class = ref($this) || $this;

    $self={
	data => {},
    };

    bless($self , $class );

    return $self;
}


sub mime_encode_base64 {
    my $self = shift;
    my ($bin_string) = @_;

#    my $b64_string = MIME::Base64::encode_base64($bin_string);
    my $b64_string = encode_base64($bin_string);

    return $b64_string;
}
    


# 		print "userCertificate;binary:: ", encode_base64( $der_cert, "" ), "\n\n";



sub convertPemCert2Der {
    my $self = shift;
    my ($pem_cert) = @_;

    my $pem_filename = './DAASIlib-Cert-tmp.pem';

    open FILE, ">$pem_filename" or 
	die "could not open file $pem_filename for write";
    binmode FILE;
    print FILE $pem_cert;
#, length($der_cert);
    close FILE;

    my $der_filename = './DAASIlib-Cert-tmp.der';

    my $command = "openssl x509 -in $pem_filename -inform PEM "
	."-outform DER -out  $der_filename";

    system($command);

    my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
	$atime,$mtime,$ctime,$blksize,$blocks) = stat $der_filename;
    my $der_cert;

    open FILE, "<$der_filename" or 
	die "could not open file $der_filename for read";
    binmode FILE;
    read FILE, $der_cert, $size;
    close FILE;


    unlink $der_filename;
    unlink $pem_filename;

    return $der_cert;
}



sub convertPemCert2Der_firstTry {
    my $self = shift;
    my ($pemcert) = @_;


    my $pem = Convert::PEM->new(
		   Name => "CERTIFICATE",
		   Macro => "Certificate",
                   ASN => qq(
			     Certificate ::= SEQUENCE  {
				 tbsCertificate		TBSCertificate,
				 signatureAlgorithm	AlgorithmIdentifier,
				 signature		BIT STRING
				 }
			     
			     TBSCertificate  ::=  SEQUENCE  {
				 version		    [0] EXPLICIT Version OPTIONAL,  --DEFAULT v1
				     serialNumber		CertificateSerialNumber,
				     signature		AlgorithmIdentifier,
				     issuer			Name,
				     validity		Validity,
				     subject			Name,
				     subjectPublicKeyInfo	SubjectPublicKeyInfo,
				     issuerUniqueID	    [1] IMPLICIT UniqueIdentifier OPTIONAL,
				     -- If present, version shall be v2 or v3
				     subjectUniqueID	    [2] IMPLICIT UniqueIdentifier OPTIONAL,
				     -- If present, version shall be v2 or v3
				     extensions	    [3] EXPLICIT Extensions OPTIONAL
				     -- If present, version shall be v3
				 }
			     
			     Version ::= INTEGER  --{  v1(0), v2(1), v3(2)  }
			     
			     CertificateSerialNumber ::= INTEGER
			     
			     Validity ::= SEQUENCE {
				 notBefore		Time,
				 notAfter		Time
				 }
			     
			     Time ::= CHOICE {
				 utcTime			UTCTime,
				 generalTime		GeneralizedTime
				 }
			     
			     UniqueIdentifier ::= BIT STRING
			     
			     SubjectPublicKeyInfo ::= SEQUENCE {
				 algorithm		AlgorithmIdentifier,
				 subjectPublicKey	BIT STRING
				 }
			     
			     Extensions ::= SEQUENCE OF Extension  --SIZE (1..MAX) OF Extension
			     
			     Extension ::= SEQUENCE {
				 extnID			OBJECT IDENTIFIER,
				 critical		BOOLEAN OPTIONAL,  --DEFAULT FALSE,
				 extnValue		OCTET STRING
				 }
			     
			     AlgorithmIdentifier ::= SEQUENCE {
				 algorithm		OBJECT IDENTIFIER,
				 parameters		ANY
				 }
			     
			     ));
    if ( ! $pem) {
	my $err = Convert::PEM->errstr; 
	print "ERROR: $err\n"; 
	die "Could not create pem-object"; 
    }
    

    my $dercert = $pem->decode(content => $pemcert);

    return $dercert;

}


1;
