#######################################################################
# Copyright: DAASI International GmbH 2018. All rights reserved.
#
# This is Open Source Software
# License: Artistic License 2.0
#
# Author: Tamim Ziai, DAASI International GmbH, www.daasi.de
# For questions please mail to tamim.ziai@daasi.de
#######################################################################
package DAASIlib::Entry;

use strict;
use warnings;
use vars qw($VERSION);
our $VERSION = "0.02";

use Log::Log4perl qw(:levels);
my $logger = Log::Log4perl->get_logger('');
use Net::LDAP::Entry;
our @ISA = ("Net::LDAP::Entry");

our $lang;

sub new {
    my $self = shift;
    $lang = shift;

    $self->SUPER::new(@_);
}

sub copy {
  my $self = shift;
  my $from = shift;
#  my $clone = $self->new($lang);

  $self->dn($from->dn());
  foreach ($from->attributes()) {
    $self->add($_ => [$from->get_value($_)]);
  }

  $self->{changetype} = $from->{changetype};
  my @changes = @{$from->{changes}};
  while (my($action, $cmd) = splice(@changes, 0, 2)) {
    my @new_cmd;
    my @cmd = @$cmd;
    while (my($type, $val) = splice(@cmd, 0, 2)) {
      push @new_cmd, $type, [ @$val ];
    }
    push @{$self->{changes}}, $action, \@new_cmd;
  }

#  $clone;
}

sub get_value {
    my $self = shift;
    my $type = lc(shift);
    my %opt  = @_;


    if ($lang) {
        my @attrs = $self->attributes();
        my $newtype = "$type;lang-$lang";
        if ( grep { lc($_) eq lc($newtype)} @attrs ) {
            $type = $newtype
        }
    }

    $self->SUPER::get_value($type, %opt);
}

sub dn {
    my $self = shift;
    my $dn = shift;
    if ($dn) {
        $dn =~ s/([^\\])\+/$1\\+/g;
        $logger->debug("dn($self): Calling super->($dn)");
        $self->SUPER::dn($dn);
    } else {
        $self->SUPER::dn();
    }
}


1;
