#######################################################################
# Copyright: DAASI International GmbH 2018. All rights reserved.
#
# This is Open Source Software
# License: Artistic License 2.0
#
# Author: Peter Gietz, DAASI International GmbH, www.daasi.de
# For questions please mail to peter.gietz@daasi.de
#######################################################################
#!/usr/bin/perl -w


package DAASIlib::Gettext;

use strict;
use warnings;
use vars qw($VERSION);


use Log::Log4perl qw(:levels);

my $logger = Log::Log4perl->get_logger("main"); 

our $VERSION = "0.10";

use DAASIlib::Data;
my $data = new DAASIlib::Data;


our %TRANSLATIONS;
our $DEFAULTLANG = 'en_EN';
our $LANGUAGE = 'de_DE';

our $ACTIVE = 0;

sub new {
    my $class;
    my $self;

    my ( $this , $param ) = @_;
    $class = ref($this) || $this;

    $self={
	data => {},
    };

    bless($self , $class );

    return $self;

}

sub init {
    my $self = shift;
    my ( $filename, $lang ) = @_;
    
    if ( ! $lang) {
	$lang = $LANGUAGE;
    }

    $lang = $self->getlangstraight($lang);
    

    
    my $msgid;
    my $msgstr;
    
    open  FH, "<$filename" or die "cannot open file $filename for read: $!";
    while ( <FH> ) {
	chomp;
	
	if ( /^msgstr "(.+)"/ ) {
	    $msgstr = $1;
	    if ( $msgstr &&  ! $msgid ) {
		print "gettext::init: Error msgstr |$msgstr| without msgid "
		    ."in file $filename\n";
	    }
	    else {
#		print "setting for $lang id: |$msgid| to string |$msgstr|\n";
		$TRANSLATIONS{$lang}{$msgid} = $msgstr;
		$msgstr = '';
		$msgid = '';
	    }
	} 
	
	if ( /^msgid "(.+)"/ ) {
	    $msgid = $1;
	}
    }
    close (FH);
    $ACTIVE = 1;
}


sub setlanguage {
    my $self = shift;
    my ( $lang ) = @_;

    $lang = $self->getlangstraight($lang);
#    print "setting lang to $lang\n";
    $LANGUAGE = $lang;
}

sub getlanguage {
    my $self = shift;
    return $LANGUAGE;
}

sub getlangstraight {
    my $self = shift;
    my ( $lang ) = @_;
   
    if ( length( $lang ) == 2 ) {
	$lang = lc($lang).'_'.uc($lang);
    }

    return $lang;
}

sub print {
    my $self = shift;
    my ( $lang ) = @_;
    print "printing |$lang|\n";
    foreach my $key ( keys %{$TRANSLATIONS{$lang}} ) {
	print "$key: $TRANSLATIONS{$lang}{$key}\n";
    }
    print "printing ended\n";

}

sub text {
    my $self = shift;

    my ( $string ) = @_;

    if ( ! $ACTIVE || $LANGUAGE eq $DEFAULTLANG ) {
	return $string;
    }
#print "gt->text() lang is $LANGUAGE\n";    
    my %langx = %{$TRANSLATIONS{$LANGUAGE}};
    if ( $langx{$string} ) {
	return ( $TRANSLATIONS{$LANGUAGE}->{$string} ); 
    }
    else {
#	print "not found\n";
	return ($string);
    }

}


1;
