#######################################################################
# Copyright: DAASI International GmbH 2018. All rights reserved.
#
# This is Open Source Software
# License: Artistic License 2.0
#
# Author: Peter Gietz, DAASI International GmbH, www.daasi.de
# For questions please mail to peter.gietz@daasi.de
#######################################################################
#!/usr/bin/perl -w

use warnings;
use strict;

# 11.11.2005 BW

package DAASIlib::HTML;

require HTML::Template;
use Log::Log4perl qw(:easy);
my $logger = Log::Log4perl->get_logger('');
use Net::LDAP::Util qw(ldap_explode_dn canonical_dn);

# print any test html page
sub printTestPage {
        my ( $template ) = @_;
        my $html_response;
        my $q = new CGI;
        $html_response =
                $q->header(
                        -type   => 'text/html',
                        -charset=> 'utf-8',
        #               -pragma => 'no-cache', # for debugging only
        #               -expires=> 'now', # for debugging only
                        -expires=> '+3m', # should be used in production
                        );

        my $test_template = HTML::Template->new(
                filename => $template,
                loop_context_vars => 1,
                die_on_bad_params => 0,
                );
         $html_response .= $test_template->output;
        return $html_response;
} # printTestPage

# print the footer of the webconnector page
sub printFooter {
        my ( $template ) = @_;
        my $html_response;
        my $q = new CGI;

        my $test_template = HTML::Template->new(
                filename => $template,
                loop_context_vars => 1,
                die_on_bad_params => 0,
                );
         $html_response = $test_template->output;
        return $html_response;
} # printFooter

# make a list of rdn values out of a dn suitable for displaying search results
sub dn2title {
        my ( $dn, $displayname, $deletemap ) = @_;
        my $newDn=undef;
        return 'World' if $dn eq '';
        my $rdns = ldap_explode_dn( $dn );
        my $dnPart;
        foreach $dnPart ( @{$rdns} ) {
            my $value = join ( "+", values %{$dnPart}  );
	    $logger->info("dn2title(): value: $value");
	    if ( $deletemap && $value =~ /$deletemap/ ) {
	    $logger->info("dn2title(): deletemap: $deletemap");
		$value =~ s/$deletemap//;
	    }

	    if ( $value ) {
		if ($newDn) {
		    $newDn .= ", ".$value;
		} else {
		    $newDn .= $displayname ? $displayname : $value;
		}
	    }
	}
        return($newDn);
}

1;
