#######################################################################
# Copyright: DAASI International GmbH 2018. All rights reserved.
#
# This is Open Source Software
# License: Artistic License 2.0
#
# Author: Peter Gietz, DAASI International GmbH, www.daasi.de
# For questions please mail to peter.gietz@daasi.de
#######################################################################
package DAASIlib::LOG;

use strict;
use warnings;
use vars qw($VERSION);

use Log::Log4perl qw(:levels :nowarn);
#use Log::Log4perl::Level;




our $logger = Log::Log4perl->get_logger("main");

our $VERSION = "0.1";
our $ISDEBUG=0;

our $OFF;

sub new
{
    my $class;
    my $self;

    my ( $this , $param ) = @_;
    $class = ref($this) || $this;

    $self={
	data => {},
    };

    bless($self , $class );

    return $self;
}


sub init {
    my $self = shift;
    my ( $progname, $is_verbose, $pid ) = @_;

    $self->{data}{progname} = $progname;
    $self->{data}{is_verbose} = $is_verbose;
    $self->{data}{pid} = $pid if $pid;


}

sub logStartmessage {
    my $self = shift;

    my $progname = $self->{data}{progname};
    $logger->error("**** $progname started ****");

    print "\n$progname started\n" if $self->{data}{is_verbose};
}

sub logAndExit {
    my $self = shift;

    my $progname = $self->{data}{progname};
    $logger->error("**** $progname ended ****");

    print "$progname ended\n" if $self->{data}{is_verbose};
    exit();
}


sub setLogLevel {
    my $self = shift;
    my ( $level ) = @_;

    my $loglevel;

    $level = lc($level);

    if ( $level eq "no" ) { $loglevel = $OFF }
    elsif ( $level eq "all" ) { $loglevel = $ALL }
    elsif ( $level eq "debug" ) { $loglevel = $DEBUG }
    elsif ( $level eq "private" ) { $loglevel = $Log::Log4perl::PRIVATE_INT }
    elsif ( $level eq "info" ) { $loglevel = $INFO }
    elsif ( $level eq "warn" ) { $loglevel = $WARN }
    elsif ( $level eq "error" ) { $loglevel = $ERROR }
    elsif ( $level eq "fatal" ) { $loglevel = $FATAL }
    else { $loglevel = $OFF }

    return ($loglevel);
}

sub initLog4PerlUsingConfig {
    my $self = shift;
    my ( $logfile ) = @_;

    Log::Log4perl::init($logfile);
    my $log = Log::Log4perl->get_logger();

    return $log;
}

sub initLog4Perl {
    my $self = shift;
    my ( $logfile, $loglevel, $is_debug ) = @_;

    use Log::Log4perl qw(:levels);
    use Log::Log4perl::Level;
    use Log::Log4perl::Appender::Screen;
    use Log::Log4perl::Appender;
#    use Log::Log4perl::Filter::LevelMatch;
#    use Log::Log4perl::Filter;

## We want an additional Loglevel for log messages containing privacy relevant
## data:
#    Log::Log4perl::Logger::create_custom_level("PRIVATE", "WARN");


    my $log = Log::Log4perl->get_logger('');

    my $layout = Log::Log4perl::Layout::PatternLayout->new("%d> %p(%P): %m (%F %L)%n");

    my $filename = $logfile;
    if ( $filename =~ m|^([\w./-]+)$| ) {
	$filename = $1;
    }
    else {
	die "\n####### Filename $filename contains invalid characters\n";
    }



    # Filter to match level Private
#    log4perl.filter.MatchPrivate = Log::Log4perl::Filter::LevelMatch
#    log4perl.filter.MatchPrivate.LevelToMatch  = PRIVATE
#    log4perl.filter.MatchPrivate.AcceptOnMatch = true
#    my $privateFilter =
#	Log::Log4perl::Filter::LevelMatch->new("Log::Log4perl::Filter",
#					       LevelToMatch => 'ERROR',
#					       LaccepTonMatch => 'true');

    # Filter to match all error and warnings level
#    log4perl.filter.MatchAllErrors       = Log::Log4perl::Filter::LevelRange
#    log4perl.filter.MatchAllErrors.LevelMin      = WARN
#    log4perl.filter.MatchAllErrors.LevelMax      = FATAL
#    log4perl.filter.MatchAllErrors.AcceptOnMatch = true

    # Filter to match all non private non errors
#    log4perl.filter.MatchAllAccess       = Log::Log4perl::Filter::LevelRange
#    log4perl.filter.MatchAllAccess.LevelMin      = DEBUG
#    log4perl.filter.MatchAllAccess.LevelMax      = INFO
#    log4perl.filter.MatchAllAccess.AcceptOnMatch = true

#    my $file_app_error =
#	Log::Log4perl::Appender->new("Log::Log4perl::Appender::File",
#				     name => "filelog",
#				     filename => "$filename.error",
#				     layout => $layout,
#				     filter => "MatchAllErrors");
#    my $file_app_access =
#	Log::Log4perl::Appender->new("Log::Log4perl::Appender::File",
#				     name => "filelog",
#				     filename => "$filename.access",
#				     layout => $layout,
#				     filter => "MatchAllAccess");
    my $file_app =
	Log::Log4perl::Appender->new("Log::Log4perl::Appender::File",
				     name => "filelog",
				     filename => $filename);


    $file_app->layout($layout);
    $log->add_appender($file_app);


#    $log->add_appender($file_app_error);
#    $log->add_appender($file_app_access);

    if ( $is_debug ) {
	my $screenlayout = Log::Log4perl::Layout::PatternLayout->new("   %m %n%n");
	my $stdout_app =
	    Log::Log4perl::Appender->new("Log::Log4perl::Appender::Screen",
					 name      => "screenlog",
					 stderr    => 0);
	$stdout_app->layout($screenlayout);
	$log->add_appender($stdout_app);
    }


    $log->level($loglevel);

    return ($log);
}




1;
