#######################################################################
# Copyright: DAASI International GmbH 2018. All rights reserved.
#
# This is Open Source Software
# License: Artistic License 2.0
#
# Author: Peter Gietz, DAASI International GmbH, www.daasi.de
# For questions please mail to peter.gietz@daasi.de
#######################################################################
package DAASIlib::Measure;


use 5.008001;
use strict;
use warnings;
use vars qw($VERSION);

$VERSION = "0.32";


use Time::HiRes qw (gettimeofday);


#require Exporter;

#our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use DAASIlib::Measure ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
#our %EXPORT_TAGS = ( 'all' => [ qw(
#	
#) ] );

#our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

#our @EXPORT = qw(
#	
#);

#our $VERSION = '0.01';


# Preloaded methods go here.


our $i =0;
our $longesttitlelen = 0;

sub new {
    my $class;
    my $self;

    my ( $this , $param ) = @_;
    $class = ref($this) || $this;

    $self={
	data => {},
    };

    bless($self , $class );

    return $self;
}


sub init {

    my $self = shift;

    $self->setstep("starting Measurement", gettimeofday());
    
}



sub setstep {
    my $self = shift;
    my ($title ) = @_;
    
    $self->{data}{timestamps}[$i] = gettimeofday();
    $self->{data}{titles}[$i++] = $title;

    if (length($title) > $longesttitlelen) {
	$longesttitlelen = length($title);
    }
    
}


sub report {
    my $self = shift;
    
    my $inc = 1;
    my $elapsed;
    my $title;
    
    while ( $self->{data}{timestamps}[$inc] ) {
	$elapsed = $self->{data}{timestamps}[$inc]- $self->{data}{timestamps}[$inc-1];
	$title = $self->{data}{titles}[$inc] ? $self->{data}{titles}[$inc]:"no title";
	printf("Step %3d: %-${longesttitlelen}s -> %12f Sek.\n",
	       $inc, $title, $elapsed);
	$inc++;
    }
    
    $elapsed = $self->{data}{timestamps}[$inc-1] - $self->{data}{timestamps}[0];      print "______________________________________________________________\n";
    printf( "          Total time:".(" "x ($longesttitlelen-10))." -> ");

    if ($elapsed > 60 ) {
	my ( $min, $sek );
	$min = int ($elapsed / 60);
	$sek = $elapsed - ($min*60);
	printf("%d Min. %10f Sek.\n", $min, $sek);

    } else { 
	printf("%12f Sek.\n", $elapsed);
    }
    

}


1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

DAASIlib::Measure - Perl extension for blah blah blah

=head1 SYNOPSIS

  use DAASIlib::Measure;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for DAASIlib::Measure, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

Peter Gietz, E<lt>zrngi01@suse.deE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by Peter Gietz

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
