#######################################################################
# Copyright: DAASI International GmbH 2018. All rights reserved.
#
# This is Open Source Software
# License: Artistic License 2.0
#
# Author: Tamim Ziai, DAASI International GmbH, www.daasi.de
# For questions please mail to tamim.ziai@daasi.de
#######################################################################
package DAASIlib::REST;

use strict;
use warnings;
use vars qw($VERSION);
our $VERSION = "0.01";

use Log::Log4perl qw(:levels);
my $logger = Log::Log4perl->get_logger('');
use JSON;
use Module::Load;

my $useragent;
my $hosturl;

#sub LWP::UserAgent::get_basic_credentials {
#    warn "@_\n";
#}


sub new {
    my $class;
    my $self;

    my ( $this , $url, $rh_options, $port, $realm, $username, $passwd, $restclass ) = @_;

    use Data::Dumper qw(Dumper);
    $logger->debug("new(): Called with port |$port|, realm |$realm|, username |$username|, options:".Dumper($rh_options) );

    $class = ref($this) || $this;

    $self = {
        data => {},
    };

    bless($self , $class );
    $restclass = 'LWP::UserAgent' if !$restclass;
    
    load $restclass;
    
    $hosturl = $url;
    $useragent = $restclass->new(%{$rh_options});
    $useragent->protocols_allowed( [ 'http', 'https'] );
    $useragent->env_proxy;
    $url =~ /^http[s]?:\/\/([^\/:]+).*$/;
    my $server = $1.":$port";
    $useragent->credentials($server, $realm, $username, $passwd);
    
    return $self;
}

sub _build_url {
    my $self = shift;
    my ($path, $pid) = @_;
    my $url = $hosturl;

    $url = _normalize($url, $path);
    $url = _normalize($url, $pid);

    return $url;
}

sub _normalize {
    my ($url, $path) = @_;
    if ($path) {
        ### Remove trailing '/' if any
        if ($url =~ /(.+)[\/]$/ ) {
            $url = $1;
        }
        ### Remove leading '/' if any
        if ($path =~ /^[\/](.+)$/ ) {
            $path = $1;
        }
        $url .= '/'.$path;
    }
    return $url;
}

sub _build_response {
    my ($response) = @_;
    my $result;
    my $data;
    if ($response->is_success) {
        $result = 0;
        $data = decode_json($response->decoded_content) if $response->decoded_content;
    } else {
        $result = $response->status_line;
    }
    
    return ($result, $data);
}

sub get {
    my $self = shift;
    my ($path, $pid) = @_;
    my $url = $self->_build_url($path, $pid);
    $logger->debug("get(): Going to get: $url" );

    my $response = $useragent->get($url);
    return _build_response($response);
}

sub post {
    my $self = shift;
    my ($path, $pid, $r_form) = @_;
    my $url = $self->_build_url($path, $pid);

    my $response = $useragent->post($url, $r_form);
    return _build_response($response);
}

sub put {
    my $self = shift;
    my ($path, $pid, $r_form) = @_;
    my $url = $self->_build_url($path, $pid);

    my $response = $useragent->put($url, $r_form);
    return _build_response($response);
}

sub delete {
    my $self = shift;
    my ($path, $pid) = @_;
    my $url = $self->_build_url($path, $pid);
    $logger->debug("delete(): Going to delete: $url" );

    my $response = $useragent->delete($url);
    return _build_response($response);
}


1;
