#######################################################################
# Copyright: DAASI International GmbH 2018. All rights reserved.
#
# This is Open Source Software
# License: Artistic License 2.0
#
# Author: Peter Gietz, DAASI International GmbH, www.daasi.de
# For questions please mail to peter.gietz@daasi.de
#######################################################################
#!/usr/bin/perl -w


package DAASIlib::SMTP;

use strict;
use warnings;
use vars qw($VERSION);
our $VERSION = "0.1";

use MIME::Base64;
use File::Basename;
use Data::Dumper qw(Dumper);

use Log::Log4perl qw(:levels);
my $logger = Log::Log4perl->get_logger(''); 
use DAASIlib::Data;
my $data = new DAASIlib::Data;


my $enhancedheader = "User-Agent: Net::SMTP\nMIME-Version: 1.0\nContent-Type: text/plain; charset=utf-8\nContent-Transfer-Encoding: 8bit\n";
my $random = $data->getRandNumString(24);
my $defaultagent = "Net::SMTP";
my $defaultcontenttype = "text/plain";
my $boundary = "--------------$random";
my $defaultmulticontenttype = "multipart/mixed;\n boundary=\"$boundary\"";
my $defaulthtmlmulticontenttype = "multipart/alternative;\n boundary=\"$boundary\"";
my $defaultcharset = "utf-8";
my $defaultencoding = "8bit";
my $defaultisstarttls = 0;
my $defaultsaslmethod = 'CRAM-MD5';

my $enhancedmailxheader = ' -a "User-Agent: Net::SMTP" -a "MIME-Version: 1.0" -a "Content-Type: text/plain; charset=utf-8" -a "Content-Transfer-Encoding: 8bit"';

use Net::SMTPS;

sub new {
    my $class;
    my $self;

    my ( $this , $param ) = @_;
    $class = ref($this) || $this;

    $self={
	data => {},
    };

    bless($self , $class );

    return $self;

}

# CHANGES
# MH 2017-07-25 set SASL Method (param no. 7)
sub set_server {
    my $self = shift;
    my ($server, $charset, $encoding, $agent, $hello, $isstarttls, $saslmethod ) = @_;

    $self->{data}{smtprelay} = $server;
    if ( $server eq 'LOCAL' | $server eq 'SYSTEM' ) {
	$self->{data}{norelay} = 1;
    }

    if ( $hello ) {
	$self->{data}{hello} = $hello;
    }

    if ( $charset ) {
	$self->{data}{charset} = $charset;
    }
    else {
	$self->{data}{charset} = $defaultcharset;
    }

    if ( $encoding ) {
	$self->{data}{encoding} = $encoding;
    }
    else {
	$self->{data}{encoding} = $defaultencoding;
    }

    if ( $agent ) {
	$self->{data}{agent} = $agent;
    }
    else {
	$self->{data}{agent} = $defaultagent;
    }

    if ( $isstarttls ) {
	$self->{data}{starttls} = $isstarttls;
    }
    else {
	$self->{data}{starttls} = $defaultisstarttls;
    }
    if ( $saslmethod ) {
	$self->{data}{saslmethod} = $saslmethod;
	# one of CRAM-MD5 (the default), LOGIN, ANONYMOUS, DIGEST, EXTERNAL, 
	# GSSAPI, PLAIN, NTLM - see perldoc Authen::SASL::Perl
    }
    else {
	$self->{data}{saslmethod} = $defaultsaslmethod;
    }

}

# CHANGES
# MH 2017-07-24 added replyto (param number 12) 
sub send_mail {
   my $self = shift;
   my ($from, $subject, $mail_data, 
       $to_mail_addresses, $cc_mail_addresses, 
       $bcc_mail_addresses, $authname, $ishtml, $attachments, 
       $createMID, $checkToAddress, $replyto ) = @_;
   my $smtp;

   $logger->debug("SMTP::send_mail(): Will try to send mail");
   $logger->debug("SMTP::send_mail(): subject: |$subject|");
   if ( ! $self->{data}{smtprelay} ) {
       $logger->error("SMTP::send_mail(): send_mail() called without a server set");
       $logger->error("SMTP::send_mail(): you have to first call set_server, before you can use method send_mail");
##       print "you have to first call set_server, before you can use method send_mail\n";
       exit();
   }

   my $contenttype;
   $subject = $self->encode_subject($subject);
   
   if ( $attachments ) {
       $contenttype = $defaultmulticontenttype;
   }
   elsif ( $ishtml )  {
       $contenttype = $defaulthtmlmulticontenttype;
   }
   else {
       $contenttype = $defaultcontenttype;
   }

#   my $hello = $data->getip();
#   my $hello = 'usha.daasi.int';

   my $header;
   my $timestamp = $self->create_timestamp();
     

   if ( $self->{data}{norelay} ) { 
       $header = ' -a "From: '.$from.'"';
       $header .= ' -a "Reply-To: <'.$replyto.'>"' if $replyto;
       $header .= ' -a "Date: '.$timestamp.'"';
       $header .= $enhancedmailxheader;

       my $adds = '';
       if ( $cc_mail_addresses ) {
	   $adds .= ' -c '.$cc_mail_addresses;
       }
       if ( $bcc_mail_addresses ) {
	   $adds .= ' -b '.$bcc_mail_addresses;
       }
       $mail_data = "\"$mail_data\"";

       my $command = "echo $mail_data | mailx -s \"$subject\" $adds $header $to_mail_addresses";
      $logger->info("SMTP::send_mail(): sending the following to system: \n$command\n");  
       system ($command);
   }
   else { # i.e. !norelay
       my @toaddresses=();
       foreach my $address ( split ( /, ?/, $to_mail_addresses  ) ) {
	   if ( $checkToAddress && $address !~/</ ) { 
	       $logger->debug("SMTP::send_mail(): Adding <...> to toadress: $address");
	       my $localpart = $address;
	       $localpart =~ s/^(.*)@(.*)/$1/;
	       push @toaddresses, '"'.$localpart.'" <'.$address.'>';
	   }
	   else {
	       push @toaddresses, $address;
	   }
       } 
       $to_mail_addresses = join(',',@toaddresses);
       
       $header = "From: $from\nTo: $to_mail_addresses\n";
       my $ssl;
       if ( $self->{data}{starttls} ) {
	   $ssl = 'starttls';
       }
     


#       $smtp = Net::SMTPS->new($self->{data}{smtprelay},
#			      Hello => $self->{data}{hello},
#	   );
       $smtp = $self->create_smtp ( $self->{data}{smtprelay}, 
			      $self->{data}{hello},
			      $ssl);

       
       if (! $smtp) {
	   $logger->error("SMTP::send_mail(): first try to connect to relay failed");
	   $logger->error("SMTP::send_mail(): will try again in 10 seconds");
	   sleep 10;
	   $smtp = &create_smtp ( $self->{data}{smtprelay}, 
				  $self->{data}{hello},
				  $ssl );

#	   $smtp = Net::SMTP->new($self->{data}{smtprelay},
#			      Hello => $self->{data}{hello},
#			      SSL => 1,
#	       );
	   if (! $smtp) {
	       $logger->error("SMTP::send_mail(): second try to connect to relay failed");
	       $logger->error("SMTP::send_mail(): will abort sending mail");
#	       print "##### ERROR couldn't open new connection to smtp server!!\n";
	       return 0;
	   }
       }
  
       my $authpw='';
       if ( $authname ) {
	   $authpw = $data->getSecret('smtpauth');
	   my $saslmethod = $self->{data}{saslmethod};
	   my $success = $smtp->auth($authname,$authpw,$saslmethod);
	   if ($success ) {
	       $logger->debug("SMTP::send_mail(): performed SMTP AUTH (SASL: $saslmethod) with user $authname");
           } else {
	       $logger->warn("SMTP::send_mail(): SMTP AUTH (SASL: $saslmethod) with user $authname failed  - wrong smtpauth value in configuration?");
           }
       }
       
       $smtp->mail($from);

       $smtp->to  ( @toaddresses );

# $smtp->to  ( split ( /,/, $to_mail_addresses  ), { Notify => ['FAILURE','DELAY'], SkipBad => 1 }); 
# könnte man noch hinten dran stellen noch nicht getestet!
       
       if ( $cc_mail_addresses ) {
	   $smtp->cc  ( split ( /,/, $cc_mail_addresses  ));
	   $header .= "Cc: $cc_mail_addresses\n";
       }
       
       if ( $bcc_mail_addresses ) {
	   $smtp->bcc ( split ( /,/, $bcc_mail_addresses ));
       }
       my $timestamp = $self->create_timestamp();
       if ( $createMID ) {
	   my $date = $data->getFileTimeStamp();
	   my $randstr = $data->getRandNumString(24);
	   my $domainpart = $self->{data}{smtprelay};
	   my $MID = '<'.$date.'.'.$randstr.'@'.$domainpart.'>';
	   $header .= "Message-ID: $MID\n";
       }

       $header .= "Reply-To: <$replyto>\n" if $replyto;
       $header .= "Date: $timestamp\n";
       $header .= "Subject: $subject"."\n";
       $header .= "User-Agent: ".$self->{data}{agent}."\n";

       $header .= "MIME-Version: 1.0\n";


       $header .= "Content-Type: $contenttype";
       if ( ! $ishtml && ! $attachments ) {
	   $header .= "; charset=".$self->{data}{charset}."\n";
	   $header .= "Content-Transfer-Encoding: ".$self->{data}{encoding}."\n\n";
	   $smtp->data($header,
		       $mail_data);
       }
       else {
	   $header .= "\n\n";
	   $header .= "This is a multi-part message in MIME format.\n";
	   my $htmlmail_data='';
	   my $cleaned_data='';
	   if ( $ishtml ) {
	       $htmlmail_data = $self->create_html_mail($mail_data); 
	       $smtp->data();
	       $smtp->datasend($header);
	       $smtp->datasend("--$boundary\n");
     	       $smtp->datasend("Content-Type: $defaultcontenttype; charset="
			       .$self->{data}{charset}."\n");
#	       $smtp->datasend("Content-Disposition: quoted-printable\n");
	       $smtp->datasend("Content-Transfer-Encoding: "
			       .$self->{data}{encoding}."\n\n");
	       $cleaned_data = $self->clean_data($mail_data);
	       $smtp->datasend($cleaned_data);
	       $smtp->datasend("--$boundary\n");
	       $smtp->datasend("Content-Type: text/html; charset="
			       .$self->{data}{charset}."\n");
	       $smtp->datasend("Content-Transfer-Encoding: "
			       .$self->{data}{encoding}."\n\n");
	       $smtp->datasend($htmlmail_data);
	       $smtp->datasend("\n--$boundary--\n");
	       $smtp->dataend();
	   }
	   else {
	       $smtp->data();
	       $smtp->datasend($header);
	       $smtp->datasend("--$boundary\n");
     	       $smtp->datasend("Content-Type: $defaultcontenttype; charset="
			       .$self->{data}{charset}."\n");
	       $smtp->datasend("Content-Transfer-Encoding: "
			       .$self->{data}{encoding}."\n\n");
	       $smtp->datasend($mail_data);
	       foreach my $attachdef ( split / ?; ?/, $attachments ) {
		   my ( $filepath, $mimetype, $isbase64 ) = split ( / ?, ?/, 
								$attachdef);
		   $logger->debug("SMTP::send_mail(): Working on Attachment $filepath");
		   unless(open INPUT, $filepath) {
		       $logger->error("SMTP::send_mail(): Unable to open input file '$filepath'");
		       exit();
		   }
		   binmode INPUT;
		   undef $/;
		   my $filecontent = <INPUT>;

		   my $encoding;

		   if ( $isbase64 ) {
		       $filecontent = encode_base64($filecontent);
		       $encoding = "base64";
		   }
		   else {
		       $encoding = $self->{data}{encoding};
		   }
		   my $filename = fileparse($filepath);
		   $smtp->datasend("\n\n--$boundary\n");
		   $smtp->datasend("Content-Type: $mimetype;\n name=\"$filename\"\n");
		   $smtp->datasend("Content-Transfer-Encoding: $encoding\n");
		   $smtp->datasend("Content-Disposition: attachment;\n filename=\"$filename\"\n");
		   $smtp->datasend("\n$filecontent")
	       }
	       $smtp->datasend("--$boundary--\n");
	       $smtp->dataend();
	   }

       }

       $smtp->quit;
   }
   return 1;

} # send_mail 

sub create_html_mail {
    my $self = shift;
    my ( $data ) = @_;

    $logger->debug("create_html_mail() started");
    my $tab_substitution = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';

    my $header = '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">'
	."\n"
	.'<html><head><meta http-equiv="Content-Type" content="text/html; charset="'
	.$self->{data}{charset}. '" />'
	."\n</head>\n\n<body>";

#    print "|$header|\n";

    $data =~ s|([ \n])\*(.*?)\*([ \n.,])|$1<b>$2</b>$3|g;
    $data =~ s|([ \n])_(.*?)_([ \n.,])|$1<u>$2</u>$3|g;
    $data =~ s|\t|$tab_substitution|g;

    if ( $data =~ m{http://} ) {
	$data =~ s{(http://.*?)([ )\n]|[\.,;:] )}{<a href="$1">$1</a>$2}g;
	$data =~ s{(https://.*?)([ )\n]|[\.,;:] )}{<a href="$1">$1</a>$2}g;

    }
    else {
	$data =~ s{([^/])(www\..*?)([ )\n]|[\.,;:] )}{$1<a href="http://$2">$2</a>$3}g;
	$data =~ s{(https://.*?)([ )\n]|[\.,;:] )}{<a href="$1">$1</a>$2}g;
    }
    $data =~ s{([\w\-\.]*?\@[\w\-\.]*?)([ )\n]|[\.,;:] )}{<a href="mailto:$1">$1</a>$2}g;
    $data =~ s|\n|<br>\n|g;
    
    $data = "$header$data</body></html>"; 

    return ( $data);
}



sub create_timestamp {
    my $self = shift;
    
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();

    my $utcdiff = 1;

    if ( $isdst ) {
	$utcdiff++;
    }

    my @monabbr = qw( Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec );
    my $monstr = $monabbr[$mon];

    my @wdayabbr = qw( Sun Mon Tue Wed Thu Fri Sat );
    my $wdaystr = $wdayabbr[$wday];

    $year+=1900;
#    my $datestring = sprintf("${wdaystr}, $mday $monstr $year ${hour}:${min}:${sec} +0${utcdiff}00";
    my $datestring = sprintf("%s, %02d %s %04d %02d:%02d:%02d +%02d00", 
			     $wdaystr,$mday,$monstr,$year,
			     $hour,$min,$sec,$utcdiff);
    return $datestring;

}

sub clean_data {
    my $self = shift;
    my ( $data ) = @_;
    $logger->debug("SMTP::clean_data(): started (data: $data");

    $data =~ s/<.*?>//g;
    $logger->debug("SMTP::clean_data(): returning $data");
    return $data;
}

## encode subject as described in RFC 2047, 2
## Format: =?charset?encoding?encoded-text?=
## (see also 
## http://ncona.com/2011/06/using-utf-8-characters-on-an-e-mail-subject/) 
sub encode_subject {
    my $self = shift;
    my ( $str ) = @_;
    $logger->debug("SMTP::encode_subject(): started with string $str");
    my $encoded;
    if ( $str =~ /[^[:ascii:]]/ ) {
	chomp($str);
	$logger->debug("SMTP::encode_subject(): contains non ascii!");
	my $charset = $self->{data}{charset};
	$encoded = encode_base64($str,'');
	chomp($encoded);
	$encoded = "=?$charset?B?$encoded?="
    }
    else {
	$encoded = $str;
    }
    $logger->debug("SMTP::encode_subject(): returning |$encoded|");
    return $encoded;
}

# CHANGES:
# MH 2017-09-04 for non-SSL, call Net::SMTP instead of Net::SMTPS. And recognizes Ports.
sub create_smtp {
    my $self = shift;
    my ( $relay, $hello, $ssl ) = @_;
    my $smtp;

    my ($host, $port);
    if ($relay =~ /[^:]+:\d+/) {
	$host = $1;
	$port = $2;
    }

    if ( $ssl ) {
	$logger->debug("SMTP::create_smtp(): SSL: $ssl");
	if ($host && $port) {
	    $smtp = Net::SMTPS->new(Host => $host,
				    Port => $port,
				    Hello => $hello,
				    doSSL => $ssl,
		);
	} else {
	    $smtp = Net::SMTPS->new(Host => $relay,
				    Hello => $hello,
				    doSSL => $ssl,
		);
	}
    }
    else {
	$logger->debug("SMTP::create_smtp(): no SSL - simply using Net::SMTP, not Net::SMTPS.");
	$smtp = Net::SMTP->new(Host => $relay,
			       Hello => $hello,
	    );
    }

    return $smtp;
}




1;
