#######################################################################
# Copyright: DAASI International GmbH 2018. All rights reserved.
#
# This is Open Source Software
# License: Artistic License 2.0
#
# Author: Peter Gietz, DAASI International GmbH, www.daasi.de
# For questions please mail to peter.gietz@daasi.de
#######################################################################
#!/usr/bin/perl -w


package DAASIlib::XML;

use strict;
use warnings;
use vars qw($VERSION);


use Log::Log4perl qw(:levels);

my $logger = Log::Log4perl->get_logger("main"); 

our $VERSION = "0.10";

our $TAB = "   ";

sub new {
    my $class;
    my $self;

    my ( $this , $param ) = @_;
    $class = ref($this) || $this;

    $self={
	data => {},
    };

    bless($self , $class );

    return $self;

}

sub startLevel {
    my $self = shift;
    my ($startlevel) = @_;
    $self->{data}{level} = $startlevel;
}

sub printArray {
    my $self = shift;
    my ($tag, $subtag, $ra_vals) = @_;
    
    $self->printStart($tag);
    foreach ( @{$ra_vals} ) {
	$self->printTag($subtag,$_);
    }
    $self->printEnd($tag);
    
}

sub printHash {
    my $self = shift;
    my ($tag, $rh_vals, $ra_order) = @_;
 
    my @keys;
    if ( $ra_order ) {
	@keys = @{$ra_order};
    }
    else {
	@keys = sort keys %{$rh_vals};
    }
    
    $self->printStart($tag);
    foreach ( @keys ) {
	if ( ref($rh_vals->{$_}) eq 'ARRAY' ) {
	    if ( @{$rh_vals->{$_}} > 1 ) {
		$self->printArray($_, 'item', $rh_vals->{$_}); 
	    }
	    else {
		$self->printTag($_, $rh_vals->{$_}[0]);
	    }
	}
	else {
	    $self->printTag($_, $rh_vals->{$_});
	}
    }
    $self->printEnd($tag);
    
}

sub print {
    my $self = shift;
    my ($string) = @_;

    if ( $string ) {
	print $TAB x $self->{data}{level}."$string\n";
    }
}

sub printTag {
    my $self = shift;
    my ($tag, $val) = @_;
    if ( $tag) {
	if ( $val ) {
	    print $TAB x $self->{data}{level}."<$tag>$val</$tag>\n";
	}
	else  {
	    print $TAB x $self->{data}{level}."</$tag>\n";
	}
    }
}

sub printStart {
    my $self = shift;
    my ($tag) = @_;

    if ( $tag) {
	print $TAB x $self->{data}{level}."<$tag>\n";
	$self->{data}{level}++;
    }
}

sub printEnd {
    my $self = shift;
    my ($tag) = @_;
    if ( $tag) {
	$self->{data}{level}--;
	print $TAB x $self->{data}{level}."</$tag>\n";
    }
}


1;
